from huobi.constant import *
from huobi.model.market import *
from huobi.utils import *
from huobi.utils.input_checker import check_in_list


class MarketClient(object):

    def __init__(self, **kwargs):
        """
        Create the request client instance.
        :param kwargs: The option of request connection.
            api_key: The public key applied from Huobi.
            secret_key: The private key applied from Huobi.
            url: The URL name like "https://api.huobi.pro".
            init_log: to init logger
        """
        self.__kwargs = kwargs

    def get_candlestick(self, symbol, period, size=200):
        """
        Get the candlestick/kline for the specified symbol. The data number is 150 as default.

        :param symbol: The symbol, like "btcusdt". To query hb10, put "hb10" at here. (mandatory)
        :param period: The candlestick/kline interval, MIN1, MIN5, DAY1 etc. (mandatory)
        :param size: The start time of of requested candlestick/kline data. (optional)
        :return: The list of candlestick/kline data.
        """
        check_symbol(symbol)
        check_should_not_none(period, "period")
        check_range(size, 1, 2000, "size")

        params = {
            "symbol": symbol,
            "period": period,
            "size": size
        }
        from huobi.service.market.get_candlestick import GetCandleStickService
        return GetCandleStickService(params).request(**self.__kwargs)

    def sub_candlestick(self, symbols: 'str', interval: 'CandlestickInterval', callback, error_handler):

        """
        Subscribe candlestick/kline event. If the candlestick/kline is updated, server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param interval: The candlestick/kline interval, MIN1, MIN5, DAY1 etc.
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(candlestick_event: 'CandlestickEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass
        :return: No return
        """

        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        check_should_not_none(interval, "interval")
        check_should_not_none(callback, "callback")

        params = {
            "symbol_list": symbol_list,
            "interval": interval,
        }

        from huobi.service.market.sub_candlestick import SubCandleStickService
        SubCandleStickService(params).subscribe(callback, error_handler, **self.__kwargs)

    def req_candlestick(self, symbols: 'str', interval: 'CandlestickInterval', callback,
                        from_ts_second=None, end_ts_second=None, error_handler=None):
        """
        Subscribe candlestick/kline event. If the candlestick/kline is updated, server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param interval: The candlestick/kline interval, MIN1, MIN5, DAY1 etc.
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(candlestick_event: 'CandlestickEvent'):
                        pass
        :param from_ts_second : data from timestamp [it's second]
        :param end_ts_second : data util timestamp [it's second]
        :param error_handler: The error handler will be called if subscription failed or error happen between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass
        :return: No return
        """

        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        check_should_not_none(interval, "interval")
        check_should_not_none(callback, "callback")

        params = {
            "symbol_list": symbol_list,
            "interval": interval,
            "from_ts_second": from_ts_second,
            "end_ts_second": end_ts_second
        }

        from huobi.service.market.req_candlestick import ReqCandleStickService
        ReqCandleStickService(params).subscribe(callback, error_handler, **self.__kwargs)

    def get_pricedepth(self, symbol: 'str', depth_type: 'str', depth_size: 'int' = None) -> PriceDepth:
        """
        Get the Market Depth of a symbol.

        :param symbol: The symbol, like "btcusdt". (mandatory)
        :param depth_type: The tpye, like "step0" to "step5". (mandatory)
        :param depth_size(optional): The maximum number of Market Depth step0 requested. range [1 - 150], default is 150
                           The maximum number of Market Depth step1,step2,step3,step4,step5 requested. size is in [5, 10, 20], default is 20.
        :return: Market Depth data.
        """

        check_symbol(symbol)
        check_in_list(depth_type, [DepthStep.STEP0, DepthStep.STEP1, DepthStep.STEP2, DepthStep.STEP3, DepthStep.STEP4,
                                   DepthStep.STEP5], "depth_type")
        params = {
            "symbol": symbol,
            "type": depth_type,
            # "depth": depth_size
        }

        from huobi.service.market.get_pricedepth import GetPriceDepthService
        ret_data = GetPriceDepthService(params).request(**self.__kwargs)

        if depth_size is not None:
            if (ret_data.bids is not None) and (len(ret_data.bids) > depth_size):
                ret_data.bids = ret_data.bids[0:depth_size]

            if (ret_data.asks is not None) and (len(ret_data.asks) > depth_size):
                ret_data.asks = ret_data.asks[0:depth_size]

        return ret_data

    @staticmethod
    def get_depth_step_list():
        return [DepthStep.STEP0,
                DepthStep.STEP1,
                DepthStep.STEP2,
                DepthStep.STEP3,
                DepthStep.STEP4,
                DepthStep.STEP5]

    @staticmethod
    def get_valid_depth_step(value, defalut_value):
        step_list = MarketClient.get_depth_step_list()
        if value in step_list:
            return value
        else:
            return defalut_value

    def sub_pricedepth(self, symbols: 'str', depth_step: 'str', callback, error_handler=None):
        """
        Subscribe price depth event. If the price depth is updated, server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param depth_step: The depth precision, string from step0 to step5.
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(price_depth_event: 'PriceDepthEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass

        :return:  No return
        """
        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        new_step = MarketClient.get_valid_depth_step(value=depth_step, defalut_value=DepthStep.STEP0)
        check_should_not_none(callback, "callback")

        params = {
            "symbol_list": symbol_list,
            "step": new_step,
        }

        from huobi.service.market.sub_pricedepth import SubPriceDepthService
        SubPriceDepthService(params).subscribe(callback, error_handler, **self.__kwargs)

    def sub_pricedepth_bbo(self, symbols: 'str', callback, error_handler=None):
        """
        Subscribe price depth event. If the price depth is updated, server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(price_depth_event: 'PriceDepthEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass

        :return:  No return
        """
        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        check_should_not_none(callback, "callback")

        params = {
            "symbol_list": symbol_list,
        }

        from huobi.service.market.sub_pricedepth_bbo import SubPriceDepthBboService
        SubPriceDepthBboService(params).subscribe(callback, error_handler, **self.__kwargs)

    def req_pricedepth(self, symbols: 'str', depth_step: 'str', callback, error_handler=None):
        """
        Subscribe price depth event. If the price depth is updated, server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param depth_step: The depth precision, string from step0 to step5.
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(price_depth_event: 'PriceDepthEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass

        :return:  No return
        """
        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        new_step = MarketClient.get_valid_depth_step(value=depth_step, defalut_value=DepthStep.STEP0)
        check_should_not_none(callback, "callback")

        params = {
            "symbol_list": symbol_list,
            "step": new_step,
        }

        from huobi.service.market.req_pricedepth import ReqPriceDepthService
        ReqPriceDepthService(params).subscribe(callback, error_handler, **self.__kwargs)

    def get_market_detail(self, symbol: 'str') -> MarketDetail:
        """
        Get trade statistics in 24 hours.

        :param symbol: The symbol, like "btcusdt". (mandatory)
        :return: Trade statistics.
        """

        check_symbol(symbol)

        params = {
            "symbol": symbol,
        }

        from huobi.service.market.get_market_detail import GetMarketDetailService
        return GetMarketDetailService(params).request(**self.__kwargs)

    def sub_market_detail(self, symbols: 'str', callback, error_handler=None):
        """
        Subscribe 24 hours trade statistics event. If statistics is generated, server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(trade_statistics_event: 'TradeStatisticsEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass
        :return:  No return
        """
        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        check_should_not_none(callback, "callback")

        params = {
            "symbol_list": symbol_list,
        }

        from huobi.service.market.sub_market_detail import SubMarketDetailService
        SubMarketDetailService(params).subscribe(callback, error_handler, **self.__kwargs)

    def req_market_detail(self, symbols: 'str', callback, error_handler=None):
        """
        Subscribe 24 hours trade statistics event. If statistics is generated, server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(trade_statistics_event: 'TradeStatisticsEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass
        :return:  No return
        """
        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        check_should_not_none(callback, "callback")

        params = {
            "symbol_list": symbol_list,
        }

        from huobi.service.market.req_market_detail import ReqMarketDetailService
        ReqMarketDetailService(params).subscribe(callback, error_handler, **self.__kwargs)

    def get_market_trade(self, symbol: 'str') -> list:
        """
        Get the most recent trades with their price, volume and direction.

        :param symbol: The symbol, like "btcusdt". (mandatory)
        :return: The list of trade.
        """

        check_symbol(symbol)

        params = {
            "symbol": symbol,
        }

        from huobi.service.market.get_market_trade import GetMarketTradeService
        return GetMarketTradeService(params).request(**self.__kwargs)

    def get_history_trade(self, symbol: 'str', size: 'int' = None) -> list:
        """
        Get the most recent trades with their price, volume and direction.

        :param symbol: The symbol, like "btcusdt". (mandatory)
        :param size: The number of historical trade requested, range [1 - 2000] (optional)
        :return: The list of trade.
        """

        check_symbol(symbol)
        check_range(size, 1, 2000, "size")

        params = {
            "symbol": symbol,
            "size": size
        }

        from huobi.service.market.get_history_trade import GetHistoryTradeService
        return GetHistoryTradeService(params).request(**self.__kwargs)

    def sub_trade_detail(self, symbols: 'str', callback, error_handler=None):
        """
        Subscribe price depth event. If the price depth is updated, server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(trade_event: 'TradeEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass
        :return:  No return
        """
        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        check_should_not_none(callback, "callback")

        params = {
            "symbol_list": symbol_list,
        }

        from huobi.service.market.sub_trade_detail import SubTradeDetailService
        SubTradeDetailService(params).subscribe(callback, error_handler, **self.__kwargs)

    def req_trade_detail(self, symbols: 'str', callback, error_handler=None):
        """
        Subscribe price depth event. If the price depth is updated, server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(trade_event: 'TradeEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass
        :return:  No return
        """
        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        check_should_not_none(callback, "callback")

        params = {
            "symbol_list": symbol_list,
        }

        from huobi.service.market.req_trade_detail import ReqTradeDetailService
        ReqTradeDetailService(params).subscribe(callback, error_handler, **self.__kwargs)

    def get_market_detail_merged(self, symbol):
        check_symbol(symbol)
        params = {
            "symbol": symbol
        }

        from huobi.service.market.get_market_detail_merged import GetMarketDetailMergedService
        return GetMarketDetailMergedService(params).request(**self.__kwargs)

    def get_market_tickers(self) -> list:
        """
        get market tickers

        :return: market ticker list.
        """

        params = {}
        from huobi.service.market.get_market_tickers import GetMarketTickersService
        return GetMarketTickersService(params).request(**self.__kwargs)

    """
    increase mbp(market by price)
    """

    def sub_mbp_increase(self, symbols: 'str', levels: 'int', callback, error_handler=None):
        """
        Subscribe mbp event. If the mbp is updated, server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param levels: level, 5，10，20，150. current only support 150
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(price_depth_event: 'PriceDepthEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass

        :return:  No return
        """
        check_should_not_none(symbols, "symbol")
        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        check_should_not_none(levels, "levels")
        check_should_not_none(callback, "callback")

        params = {
            "symbol_list": symbol_list,
            "levels": levels
        }

        from huobi.service.market.sub_mbp_increase import SubMbpIncreaseService
        SubMbpIncreaseService(params).subscribe(callback, error_handler, **self.__kwargs)

    """
    subscribe full mbp(market by price)
    """

    def sub_mbp_full(self, symbols: 'str', levels: 'int', callback, error_handler=None):
        """
        Subscribe full mbp event. If the mbp is updated, server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param levels: level, 5，10，20
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(price_depth_event: 'PriceDepthEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass

        :return:  No return
        """
        check_should_not_none(symbols, "symbol")
        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        check_should_not_none(levels, "levels")
        check_in_list(levels, [MbpLevel.MBP5, MbpLevel.MBP10, MbpLevel.MBP20], "levels")
        check_should_not_none(callback, "callback")

        params = {
            "symbol_list": symbol_list,
            "levels": levels
        }

        from huobi.service.market.sub_mbp_full import SubMbpFullService
        SubMbpFullService(params).subscribe(callback, error_handler, **self.__kwargs)

    def req_mbp(self, symbols: 'str', levels: 'int', callback, auto_close=True, error_handler=None):
        """
        Subscribe mbp event. If the mbp is updated, server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param levels: level, 5，10，20，150. current only support 150
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(price_depth_event: 'PriceDepthEvent'):
                        pass
        :param auto_close : close websocket connection after get data
        :param error_handler: The error handler will be called if subscription failed or error happen between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass

        :return:  No return
        """
        check_should_not_none(symbols, "symbol")
        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        check_should_not_none(levels, "levels")
        check_should_not_none(callback, "callback")
        params = {
            "symbol_list": symbol_list,
            "levels": levels
        }
        from huobi.service.market.req_mbp import ReqMbpService
        ReqMbpService(params).subscribe(callback, error_handler, **self.__kwargs)



from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant import *
from huobi.model.trade import *
from huobi.utils import *


class GetMatchResultsService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/order/matchresults"

        def parse(dict_data):
            data_list = dict_data.get("data", [])
            return default_parse_list_dict(data_list, MatchResult, [])

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)









from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant import *
from huobi.model.trade import *
from huobi.utils import *


class GetOrderByIdService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        order_id = self.params["order_id"]
        def get_channel():
            path = "/v1/order/orders/{}"
            return path.format(order_id)

        def parse(dict_data):
            data_dict = dict_data.get("data")
            return Order.json_parse(data_dict)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, get_channel(), self.params, parse)









from huobi.model.wallet.deposit_history_item import DepositHistoryItem
class DepositHistory:
    """
    The deposit history

    :member
        nextId: next id.
        data: history list.
    """

    def __init__(self):
        self.data = list()
        self.nextId = 0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.nextId, format_data + "NextId")
        if self.data and len(self.data):
            for item in self.data:
                item.print_object()
                PrintBasic.print_basic("", format_data + "")



import time


from huobi.connection.websocket_req_client import *
from huobi.model.trade import *
from huobi.utils import *


class ReqOrderListService:
    def __init__(self, params):
        self.params = params

    def subscribe(self, callback, error_handler, **kwargs):
        symbol = self.params["symbol"]
        account_id = self.params["account-id"]
        order_states = self.params["states"]
        client_req_id = self.params["client-req-id"]

        def subscription(connection):
            connection.send(request_order_list_channel(symbol=symbol, account_id=account_id, states_str=order_states, client_req_id=client_req_id, more_key=self.params))

        def parse(dict_data):
            order_update_event = default_parse(dict_data, OrderListReq, OrderListItem)

            return order_update_event

        WebSocketReqClient(**kwargs).execute_subscribe_v1(subscription,
                                            parse,
                                            callback,
                                            error_handler,
                                            is_trade=True)










import asyncio
import time

from huobi.client.account import AccountClient, AccountBalance, get_default_server_url, AccountType


class AccountClientPerformance(AccountClient):

    def __init__(self, **kwargs):
        """
        Create the request client instance.
        :param kwargs: The option of request connection.
            api_key: The public key applied from Huobi.
            secret_key: The private key applied from Huobi.
            url: The URL name like "https://api.huobi.pro".
            init_log: to init logger
        """

        self.__kwargs = kwargs
        self.__kwargs["performance_test"] = True
        super(AccountClientPerformance, self).__init__(**self.__kwargs)

    def get_account_balance(self) -> list:
        from huobi.service.account.get_balance import GetBalanceService
        """
        Get the balance of a all accounts.

        :return: The information of all account balance.
        """
        server_url = get_default_server_url(self.__kwargs.get("url"))
        tasks = []
        accounts, req_cost_1, cost_manual_1  = super(AccountClientPerformance, self).get_accounts()
        start_time = time.time()
        account_balance_list = []
        account_balance_json_map = {}
        for account_item in accounts:
            balance_params = {"account-id": account_item.id}
            balance_request = GetBalanceService(balance_params).get_request(**self.__kwargs)
            balance_url = server_url + balance_request.url
            tasks.append(asyncio.ensure_future(
                super(AccountClientPerformance, self).async_get_account_balance(balance_url, account_item.id, account_balance_json_map)))

        loop = asyncio.get_event_loop()
        try:
            loop.run_until_complete(asyncio.wait(tasks))
        except Exception as ee:
            print(ee)
        finally:
            # loop.close()  #for thread safe, the event loop can't be closed
            pass

        for account_id, account_balance_json in account_balance_json_map.items():
            account_balance = AccountBalance.json_parse(account_balance_json.get("data", {}))
            account_balance_list.append(account_balance)

        del account_balance_json_map
        del tasks
        end_time = time.time()
        async_balane_cost = round(end_time - start_time, 6)
        return account_balance_list, req_cost_1 + async_balane_cost, cost_manual_1 + async_balane_cost

    def get_account_by_type_and_symbol(self, account_type, symbol):
        accounts, req_cost, cost_manual = super(AccountClientPerformance, self).get_accounts()
        if accounts and len(accounts):
            for account_obj in accounts:
                if account_obj.type == account_type:
                    if account_type == AccountType.MARGIN:
                        if symbol == account_obj.subtype:
                            return account_obj, req_cost, cost_manual
                    else:
                        return account_obj, req_cost, cost_manual

        return None, req_cost, cost_manual




from huobi.constant import *


class MarginBalanceDetail:
    def __init__(self):
        self.id = 0
        self.symbol = 0
        self.state = AccountState.INVALID
        self.type = AccountType.INVALID
        self.risk_rate = 0.0
        self.fl_price = 0.0
        self.fl_type = ""
        self.sub_account_balance_list = list()

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "ID")
        PrintBasic.print_basic(self.type, format_data + "Account Type")
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        PrintBasic.print_basic(self.state, format_data + "Account State")
        PrintBasic.print_basic(self.fl_price, format_data + "Burst Price")
        PrintBasic.print_basic(self.fl_type, format_data + "Burst Type")
        PrintBasic.print_basic(self.risk_rate, format_data + "Risk Rate")



from huobi.utils import *

from huobi.connection.websocket_req_client import *
from huobi.model.account import *



class ReqAccountBalanceService:
    def __init__(self, params):
        self.params = params

    def subscribe(self, callback, error_handler, **kwargs):
        client_req_id = self.params["client_req_id"]
        def subscription(connection):
            connection.send(request_account_list_channel(client_req_id))

        def parse(dict_data):
            req_obj = AccountBalanceReq()
            req_obj.ts = dict_data.get("ts", 0)
            req_obj.topic = dict_data.get("topic", 0)
            req_obj.cid = dict_data.get("cid", 0)
            data_list = dict_data.get("data", [])
            req_obj.data = AccountBalance.json_parse_list(data_list)
            return req_obj

        WebSocketReqClient(**kwargs).execute_subscribe_v1(subscription,
                                            parse,
                                            callback,
                                            error_handler,
                                            is_trade=True)








class ConnectionState:
    IDLE = 0
    CONNECTED = 1
    WAIT_RECONNECT = 2
    CLOSED_ON_ERROR = 3
    CLOSED = 4

CONNECT_HEART_BEAT_LIMIT_MS = 60000   # max interval between two package
RECONNECT_AFTER_TIME_MS = 63000         # if not need connect immediately, it will enter delay connect status and will connect after setting times



from huobi.model.market.pricedepth import PriceDepth

class PriceDepthReq:
    """
    The price depth information.

    :member
        ts: The UNIX formatted timestamp in UTC.
        version:
        bids: The list of the bid depth. The content is DepthEntry class.
        asks: The list of the ask depth. The content is DepthEntry class.

    """
    def __init__(self):
        self.rep = ""
        self.data = PriceDepth()

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.rep, format_data + "Channel")
        self.data.print_object("\t")


from huobi.model.market.market_detail import MarketDetail

class MarketDetailEvent:
    """
     The 24H trade statistics received by subscription of trade statistics.

     :member
         Channel: topic you subscribe, include symbol.
         timestamp: The UNIX formatted timestamp generated by server in UTC.
         trade_statistics: The trade statistics.
     """

    def __init__(self):
        self.ch = ""
        self.ts = 0
        self.tick = MarketDetail()

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.ch, format_data + "Channel")
        PrintBasic.print_basic(self.ts, format_data + "Timestamp")
        self.tick.print_object()


from huobi.client.margin import MarginClient
from huobi.constant import *


margin_client = MarginClient(api_key=g_api_key, secret_key=g_secret_key)
account_balance = margin_client.get_cross_margin_account_balance()
account_balance.print_object()





from huobi.client.etf import EtfClient
from huobi.constant import *

etf_client = EtfClient(api_key=g_api_key, secret_key=g_secret_key)
etf_swap_ret = etf_client.post_etf_swap_out("hb10", 1000)
etf_swap_ret.print_object()



















from huobi.client.wallet import WalletClient
from huobi.constant import *
from huobi.utils import *


# case 1: TRX withdraw and cancel
wallet_client = WalletClient(api_key=g_api_key, secret_key=g_secret_key)
withdraw_id = wallet_client.post_create_withdraw(address="xxxxxx",
                                                     amount=40, currency="trx", fee=1,
                                                     chain=None, address_tag=None)
LogInfo.output("Create Withdraw ID {id}".format(id=withdraw_id))

withdraw_id_ret = wallet_client.post_cancel_withdraw(withdraw_id=withdraw_id)
LogInfo.output("Cancel Withdraw Id {id} {response_id}".format(id=withdraw_id, response_id=withdraw_id_ret))


# case 2: USDT withdraw and cancel
wallet_client = WalletClient(api_key=g_api_key, secret_key=g_secret_key)
withdraw_id = wallet_client.post_create_withdraw(address="xxxxxx",
                                                     amount=2, currency="usdt", fee=0,
                                                     chain="trc20usdt", address_tag=None)

withdraw_id_ret = wallet_client.post_cancel_withdraw(withdraw_id=withdraw_id)
LogInfo.output("Cancel Withdraw {withdraw_id}, {response_id}".format(withdraw_id=withdraw_id, response_id=withdraw_id_ret))








import threading
import websocket
import gzip
import ssl
import logging
import urllib.parse

from huobi.constant import *
from huobi.utils import *
from huobi.exception.huobi_api_exception import HuobiApiException
from huobi.connection.impl.private_def import ConnectionState

# Key: original_connection, Value: connection
websocket_connection_handler = dict()


def on_message(original_connection, message):
    websocket_connection = websocket_connection_handler[original_connection]
    websocket_connection.on_message(message)
    return


def on_error(original_connection, error):
    websocket_connection = websocket_connection_handler[original_connection]
    websocket_connection.on_failure(error)


def on_close(original_connection):
    websocket_connection = websocket_connection_handler[original_connection]
    websocket_connection.on_close()


def on_open(original_connection):
    websocket_connection = websocket_connection_handler[original_connection]
    websocket_connection.on_open(original_connection)


connection_id = 0


def websocket_func(*args):
    try:
        websocket_manage = args[0]
        websocket_manage.original_connection = websocket.WebSocketApp(websocket_manage.url,
                                                        on_message=on_message,
                                                        on_error=on_error,
                                                        on_close=on_close)
        global websocket_connection_handler
        websocket_connection_handler[websocket_manage.original_connection] = websocket_manage
        websocket_manage.logger.info("[Sub][" + str(websocket_manage.id) + "] Connecting...")
        websocket_manage.original_connection.on_open = on_open
        websocket_manage.original_connection.run_forever(sslopt={"cert_reqs": ssl.CERT_NONE})
        websocket_manage.logger.info("[Sub][" + str(websocket_manage.id) + "] Connection event loop down")
        if websocket_manage.state == ConnectionState.CONNECTED:
            websocket_manage.state = ConnectionState.IDLE
    except Exception as ex:
        print(ex)

class WebsocketManage:

    def __init__(self, api_key, secret_key, uri, request):
        self.__thread = None
        self.__market_url = HUOBI_WEBSOCKET_URI_PRO + "/ws"
        self.__trading_url = HUOBI_WEBSOCKET_URI_PRO + "/ws/" + request.api_version
        self.__mbp_feed_url = HUOBI_WEBSOCKET_URI_PRO + "/feed"
        self.__api_key = api_key
        self.__secret_key = secret_key
        self.request = request
        self.reconnect_at = 0
        self.original_connection = None
        self.last_receive_time = 0
        self.logger = logging.getLogger("huobi-client")
        self.state = ConnectionState.IDLE
        global connection_id
        connection_id += 1
        self.id = connection_id
        host = urllib.parse.urlparse(uri).hostname
        if host.find("api") == 0:
            self.__market_url = "wss://" + host + "/ws"
            self.__mbp_feed_url = "wss://" + host + "/feed"
            self.__trading_url = "wss://" + host + "/ws/" + request.api_version
        else:
            self.__market_url = "wss://" + host + "/api/ws"
            self.__mbp_feed_url = "wss://" + host + "/feed"
            self.__trading_url = "wss://" + host + "/ws/" + request.api_version

        if request.is_trading:
            self.url = self.__trading_url
        elif request.is_mbp_feed:
            self.url = self.__mbp_feed_url
        else:
            self.url = self.__market_url

    def close_and_wait_reconnect(self, delay_in_ms):
        if self.original_connection is not None:
            self.original_connection.close()
            self.original_connection = None
            self.state = ConnectionState.WAIT_RECONNECT
            self.reconnect_at =  + delay_in_ms
        self.logger.warning("[Sub][%d] Lost connectiong for %d ms, will try reconnecting " % (self.id, self.reconnect_at))

    def re_connect(self):
        if get_current_timestamp() > self.reconnect_at:
            self.logger.info("[Sub][%d] Reconnecting ... " % self.id)
            self.connect()

    def connect(self):
        if self.state == ConnectionState.CONNECTED:
            self.logger.info("[Sub][" + str(self.id) + "] Already connected")
        else:
            self.__thread = threading.Thread(target=websocket_func, args=[self])
            self.__thread.start()

    def send(self, data):
        # print("Send Data : " + data)
        self.original_connection.send(data)

    def close(self):
        self.original_connection.close()
        del websocket_connection_handler[self.original_connection]
        self.state = ConnectionState.CLOSED
        self.logger.info("[Sub][" + str(self.id) + "] Closing normally")

    def on_open(self, original_connection):
        self.logger.info("[Sub][" + str(self.id) + "] Connected to server")
        self.original_connection = original_connection
        self.last_receive_time = get_current_timestamp()
        self.state = ConnectionState.CONNECTED
        if self.request.is_trading:
            try:
                if self.request.api_version == ApiVersion.VERSION_V1:
                    builder = UrlParamsBuilder()
                    create_signature(self.__api_key, self.__secret_key,
                                     "GET", self.url, builder)
                    builder.put_url("op", "auth")
                    self.send(builder.build_url_to_json())
                elif self.request.api_version == ApiVersion.VERSION_V2:
                    builder = UrlParamsBuilder()
                    create_signature_v2(self.__api_key, self.__secret_key,
                                     "GET", self.url, builder)
                    self.send(builder.build_url_to_json())
                else:
                    self.on_error("api version for create the signature fill failed")

            except Exception as e:
                self.on_error("Unexpected error when create the signature: " + str(e))
        else:
            if self.request.subscription_handler is not None:
                self.request.subscription_handler(self)
        return

    def on_error(self, error_message):
        if self.request.error_handler is not None:
            exception = HuobiApiException(HuobiApiException.SUBSCRIPTION_ERROR, error_message)
            self.request.error_handler(exception)
        self.logger.error("[Sub][" + str(self.id) + "] " + str(error_message))

    def on_failure(self, error):
        self.on_error("Unexpected error: " + str(error))
        self.close_on_error()

    def on_message(self, message):
        self.last_receive_time = get_current_timestamp()
        if isinstance(message, (str)): # V2
            # print("RX string : ", message)
            dict_data = json.loads(message)
        elif isinstance(message, (bytes)): # V1
            # print("RX bytes: " + gzip.decompress(message).decode("utf-8"))
            dict_data = json.loads(gzip.decompress(message).decode("utf-8"))
        else:
            print("RX unknow type : ", type(message))
            return

        status_outer = dict_data.get("status", "")
        err_code_outer = dict_data.get("err-code", 0)
        op_outer = dict_data.get("op", "")
        action_outer = dict_data.get("action", "")
        ch_outer = dict_data.get("ch", "")
        rep_outer = dict_data.get("rep", "")
        ping_market_outer = int(dict_data.get("ping", 0))
        if status_outer and len(status_outer) and status_outer != "ok":
            error_code = dict_data.get("err-code", "Unknown error")
            error_msg = dict_data.get("err-msg", "Unknown error")
            self.on_error(error_code + ": " + error_msg)
        elif err_code_outer and int(err_code_outer) != 0:
            error_code = dict_data.get("err-code", "Unknown error")
            error_msg = dict_data.get("err-msg", "Unknown error")
            self.on_error(error_code + ": " + error_msg)
        elif op_outer and len(op_outer): # for V1
            if op_outer == "notify":
                self.__on_receive(dict_data)
            elif op_outer == "ping":
                #print("******** receive trade ping pong ********", dict_data)
                ping_ts = dict_data.get("ts", 0)
                self.__process_ping_on_trading_line(ping_ts)
            elif op_outer == "auth":
                if self.request.subscription_handler is not None:
                    self.request.subscription_handler(self)
            elif op_outer == "req":
                self.__on_receive(dict_data)
        elif action_outer and len(action_outer):  # for V2
            if action_outer == "ping":
                action_data = dict_data.get("data")
                ping_ts = action_data.get("ts")
                self.__process_ping_on_v2_trade(ping_ts)
            elif action_outer == "sub":
                action_code = dict_data.get("code", -1)
                if action_code == 200:
                    logging.info("subscribe ACK received")
                else:
                    logging.error("receive error data : " + message)
            elif action_outer == "req": #
                action_code = dict_data.get("code", -1)
                if action_code == 200:
                    logging.info("signature ACK received")
                    if self.request.subscription_handler is not None:
                        self.request.subscription_handler(self)
                else:
                    logging.error("receive error data : " + message)
            elif action_outer == "push":
                action_data = dict_data.get("data")
                if action_data:
                    self.__on_receive(dict_data)
                else:
                    logging.error("receive error push data : " + message)

        elif ch_outer and len(ch_outer):
            self.__on_receive(dict_data)
        elif rep_outer and len(rep_outer):
            self.__on_receive(dict_data)
        elif ping_market_outer:
            #print("******** receive market ping pong ********", dict_data)
            ping_ts = ping_market_outer
            self.__process_ping_on_market_line(ping_ts)
        else:
            #print("unknown data process, RX: ", gzip.decompress(message).decode("utf-8"))
            pass

    def __on_receive(self, dict_data):
        res = None
        try:
            if self.request.json_parser is not None:
                res = self.request.json_parser(dict_data)
        except Exception as e:
            self.on_error("Failed to parse server's response: " + str(e))

        try:
            if self.request.update_callback is not None:
                self.request.update_callback(res)
        except Exception as e:
            self.on_error("Process error: " + str(e)
                     + " You should capture the exception in your error handler")

        # websocket request will close the connection after receive
        if self.request.auto_close:
            self.close()

    def __process_ping_on_trading_line(self, ping_ts):
        #print("### __process_ping_on_trading_line ###")
        #self.send("{\"op\":\"pong\",\"ts\":" + str(get_current_timestamp()) + "}")
        PrintBasic.print_basic(ping_ts, "response time")
        self.send("{\"op\":\"pong\",\"ts\":" + str(ping_ts) + "}")
        return

    def __process_ping_on_market_line(self, ping_ts):
        #print("### __process_ping_on_market_line ###")
        #self.send("{\"pong\":" + str(get_current_timestamp()) + "}")
        PrintBasic.print_basic(ping_ts, "response time")
        self.send("{\"pong\":" + str(ping_ts) + "}")
        return

    def __process_ping_on_v2_trade(self, ping_ts):
        # PrintDate.timestamp_to_date(ping_ts)
        self.send("{\"action\": \"pong\",\"data\": {\"ts\": " + str(ping_ts) +"}}")
        return

    def close_on_error(self):
        if self.original_connection is not None:
            self.original_connection.close()
            self.state = ConnectionState.CLOSED_ON_ERROR
            self.logger.error("[Sub][" + str(self.id) + "] Connection is closing due to error")



import logging

from huobi.connection.impl.websocket_watchdog import WebSocketWatchDog
from huobi.connection.impl.websocket_manage import WebsocketManage
from huobi.connection.impl.websocket_request import WebsocketRequest
from huobi.constant.system import WebSocketDefine, ApiVersion


class SubscribeClient(object):
    # static property
    subscribe_watch_dog = WebSocketWatchDog()

    def __init__(self, **kwargs):
        """
        Create the subscription client to subscribe the update from server.

        :param kwargs: The option of subscription connection.
            api_key: The public key applied from Huobi.
            secret_key: The private key applied from Huobi.
            url: Set the URI for subscription.
            init_log: to init logger
        """
        self.__api_key = kwargs.get("api_key", None)
        self.__secret_key = kwargs.get("secret_key", None)
        self.__uri = kwargs.get("url", WebSocketDefine.Uri)
        self.__init_log = kwargs.get("init_log", None)
        if self.__init_log and self.__init_log:
            logger = logging.getLogger("huobi-client")
            logger.setLevel(level=logging.INFO)
            handler = logging.StreamHandler()
            handler.setFormatter(logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s'))
            logger.addHandler(handler)

        self.__websocket_manage_list = list()

    def __create_websocket_manage(self, request):
        manager = WebsocketManage(self.__api_key, self.__secret_key, self.__uri, request)
        self.__websocket_manage_list.append(manager)
        manager.connect()
        SubscribeClient.subscribe_watch_dog.on_connection_created(manager)

    def create_request(self, subscription_handler, parse, callback, error_handler, is_trade, is_mbp_feed=False):
        request = WebsocketRequest()
        request.subscription_handler = subscription_handler
        request.is_trading = is_trade
        request.is_mbp_feed = is_mbp_feed
        request.auto_close = False  # subscribe need connection. websocket request need close request.
        request.json_parser = parse
        request.update_callback = callback
        request.error_handler = error_handler
        return request

    def create_request_v1(self, subscription_handler, parse, callback, error_handler, is_trade=False):
        request = self.create_request(subscription_handler=subscription_handler, parse=parse, callback=callback,
                                      error_handler=error_handler, is_trade=is_trade)
        request.api_version = ApiVersion.VERSION_V1
        return request

    def create_request_v2(self, subscription_handler, parse, callback, error_handler, is_trade=False):
        request = self.create_request(subscription_handler=subscription_handler, parse=parse, callback=callback,
                                      error_handler=error_handler, is_trade=is_trade)
        request.api_version = ApiVersion.VERSION_V2
        return request

    def execute_subscribe_v1(self, subscription_handler, parse, callback, error_handler, is_trade=False):
        request = self.create_request_v1(subscription_handler, parse, callback, error_handler, is_trade)
        self.__create_websocket_manage(request)

    def execute_subscribe_v2(self, subscription_handler, parse, callback, error_handler, is_trade=False):
        request = self.create_request_v2(subscription_handler, parse, callback, error_handler, is_trade)
        self.__create_websocket_manage(request)

    def execute_subscribe_mbp(self, subscription_handler, parse, callback, error_handler, is_trade=False,
                              is_mbp_feed=True):
        request = self.create_request(subscription_handler, parse, callback, error_handler, is_trade, is_mbp_feed)
        self.__create_websocket_manage(request)

    def unsubscribe_all(self):
        for websocket_manage in self.__websocket_manage_list:
            SubscribeClient.subscribe_watch_dog.on_connection_closed(websocket_manage)
            websocket_manage.close()
        self.__websocket_manage_list.clear()



import json
from huobi.utils.time_service import get_current_timestamp

def dict_add_new(old_dict, new_dict):
    if old_dict  is None:
        old_dict = {}

    if new_dict and len(new_dict):
        for key_val, val in new_dict.items():
            if val:
                exist_val = old_dict.get(key_val, None)
                if exist_val and len(str(exist_val)):
                    pass
                else:
                    old_dict[key_val] = str(val)

    return old_dict

def request_kline_channel(symbol, interval, from_ts_second = None, to_ts_second = None):
    channel = dict()
    channel["req"] = "market." + symbol + ".kline." + interval
    channel["id"] = str(get_current_timestamp())
    if from_ts_second:
        channel["from"] = int(from_ts_second)
    if to_ts_second:
        channel["to"] = int(to_ts_second)
    return json.dumps(channel)


def request_trade_detail_channel(symbol):
    channel = dict()
    channel["req"] = "market." + symbol + ".trade.detail"
    channel["id"] = str(get_current_timestamp())
    return json.dumps(channel)


def request_price_depth_channel(symbol, step_type = "step0"):
    channel = dict()
    channel["req"] = "market." + symbol + ".depth." + step_type
    channel["id"] = str(get_current_timestamp())
    return json.dumps(channel)

def request_market_detail_channel(symbol):
    channel = dict()
    channel["req"] = "market." + symbol + ".detail"
    channel["id"] = str(get_current_timestamp())
    return json.dumps(channel)

def request_account_list_channel(client_req_id = None):
    channel = dict()
    channel["op"] = "req"
    channel["topic"] = "accounts.list"
    channel["cid"] = str(client_req_id) if client_req_id else str(get_current_timestamp())
    return json.dumps(channel)

def request_order_list_channel(symbol, account_id, states_str= None, client_req_id = None, more_key={}):
    channel = dict()
    try:
        channel["op"] = "req"
        channel["account-id"] = account_id
        channel["topic"] = "orders.list"
        channel["symbol"] = symbol
        if states_str and len(states_str):
            channel["states"] = str(states_str)
        channel["cid"] = str(client_req_id) if client_req_id else str(get_current_timestamp())
        channel = dict_add_new(channel, more_key)

    except Exception as e:
        print(e)
    return json.dumps(channel)

def request_order_detail_channel(order_id, client_req_id = None):
    channel = dict()
    channel["op"] = "req"
    channel["topic"] = "orders.detail"
    channel["order-id"] = str(order_id)
    channel["cid"] = str(client_req_id) if client_req_id else str(get_current_timestamp())
    return json.dumps(channel)




from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant import *
from huobi.model.margin.general_repay_loan_record import GeneralRepayLoanRecord
from huobi.model.margin.general_repay_loan_result import GeneralRepayLoanResult
from huobi.utils.json_parser import  default_parse_list_dict


class GetGeneralRepaymentLoanRecordsService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):

        def get_channel():
            path = "/v2/account/repayment"
            return path

        def parse(dict_data):
            return default_parse_list_dict(dict_data.get("data", {}), GeneralRepayLoanRecord)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, get_channel(), self.params, parse)


from huobi.client.trade import TradeClient
from huobi.constant import *
from huobi.utils import *


trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key)
symbol_test = "eosusdt"


order_id = trade_client.create_order(symbol=symbol_test, account_id=g_account_id, order_type=OrderType.BUY_LIMIT, source=OrderSource.API, amount=18, price=0.292)
LogInfo.output("created order id : {id}".format(id=order_id))
canceled_order_id  = trade_client.cancel_order(symbol_test, order_id)
if canceled_order_id == order_id:
    LogInfo.output("cancel order {id} done".format(id=canceled_order_id))
else:
    LogInfo.output("cancel order {id} fail".format(id=canceled_order_id))



from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant.system import HttpMethod
from huobi.model.account import *


class GetAccountBalanceBySubUidService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        sub_uid = self.params["sub-uid"]

        def get_channel():
            path = "/v1/account/accounts/{}"
            return path.format(sub_uid)

        def parse(dict_data):
            data_list = dict_data.get("data", [])
            return AccountBalance.json_parse_list(data_list)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, get_channel(), self.params, parse)




from huobi.client.account import AccountClient
from huobi.constant import *
from huobi.utils import *

account_client = AccountClient(api_key=g_api_key,
                              secret_key=g_secret_key)
transfer_order_id = account_client.transfer_between_parent_and_subuser(sub_uid=g_sub_uid, currency="usdt", amount=10, transfer_type=TransferMasterType.OUT)
LogInfo.output("transfer from master to subuser Order Id : {id}".format(id=transfer_order_id))

transfer_order_id = account_client.transfer_between_parent_and_subuser(sub_uid=g_sub_uid, currency="usdt", amount=10, transfer_type=TransferMasterType.IN)
LogInfo.output("transfer from subuser to master Order Id : {id}".format(id=transfer_order_id))






from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant.system import HttpMethod
from huobi.model.account import Balance
from huobi.model.margin import *
from huobi.utils import *



class GetMarginAccountBalanceService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/margin/accounts/balance"

        def parse(dict_data):
            data_list = dict_data.get("data", [])
            account_balance_list = []
            if data_list and len(data_list):
                for row in data_list:
                    account_balance = default_parse(row, MarginAccountBalance, Balance)
                    account_balance_list.append(account_balance)
            return account_balance_list

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)










class TradeMarket:
    """
    The trade information with price and amount etc.

    :member
        subUid: sub user ID.
        accountType:
        activation: sub user account state for given accountType.
    """

    def __init__(self):
        self.sub_uid = ""
        self.account_type = ""
        self.activation = ""

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.sub_uid, format_data + "subUid")
        PrintBasic.print_basic(self.account_type, format_data + "accountType")
        PrintBasic.print_basic(self.activation, format_data + "activation")



from huobi.client.account import AccountClient
from huobi.client.trade import TradeClient
from huobi.constant import *
from huobi.utils import *


symbol = "eosusdt"

trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key)
account_client = AccountClient(api_key=g_api_key,
                              secret_key=g_secret_key)

account_spot = account_client.get_account_by_type_and_symbol(account_type=AccountType.SPOT, symbol=None)
account_id_test = account_spot.id

direct_tmp = QueryDirection.NEXT
LogInfo.output("==============test case 1 for {direct}===============".format(direct=direct_tmp))
list_obj = trade_client.get_open_orders(symbol=symbol, account_id=account_id_test, direct=direct_tmp)
LogInfo.output_list(list_obj)

direct_tmp = QueryDirection.PREV
LogInfo.output("==============test case 2 for {direct}===============".format(direct=direct_tmp))
list_obj = trade_client.get_open_orders(symbol=symbol, account_id=account_id_test, direct=direct_tmp)
LogInfo.output_list(list_obj)



from huobi.client.market import MarketClient
from huobi.constant import DepthStep
from huobi.model.market import PriceDepth
from huobi.utils.input_checker import *


class MarketClientPerformance(MarketClient):

    def __init__(self, **kwargs):
        """
        Create the request client instance.
        :param kwargs: The option of request connection.
            api_key: The public key applied from Huobi.
            secret_key: The private key applied from Huobi.
            url: The URL name like "https://api.huobi.pro".
            init_log: to init logger
        """

        self.__kwargs = kwargs
        self.__kwargs["performance_test"] = True
        super(MarketClientPerformance, self).__init__(**self.__kwargs)

    def get_pricedepth(self, symbol: 'str', depth_type: 'str', depth_size: 'int' = None) -> PriceDepth:
        """
        Get the Market Depth of a symbol.

        :param symbol: The symbol, like "btcusdt". (mandatory)
        :param depth_type: The tpye, like "step0" to "step5". (mandatory)
        :param depth_size(optional): The maximum number of Market Depth step0 requested. range [1 - 150], default is 150
                           The maximum number of Market Depth step1,step2,step3,step4,step5 requested. size is in [5, 10, 20], default is 20.
        :return: Market Depth data.
        """

        check_symbol(symbol)
        check_in_list(depth_type, [DepthStep.STEP0, DepthStep.STEP1, DepthStep.STEP2, DepthStep.STEP3, DepthStep.STEP4,
                                   DepthStep.STEP5], "depth_type")
        params = {
            "symbol": symbol,
            "type": depth_type,
            # "depth": depth_size
        }

        from huobi.service.market.get_pricedepth import GetPriceDepthService
        ret_data, req_cost, cost_manual = GetPriceDepthService(params).request(**self.__kwargs)

        if depth_size is not None:
            if (ret_data.bids is not None) and (len(ret_data.bids) > depth_size):
                ret_data.bids = ret_data.bids[0:depth_size]

            if (ret_data.asks is not None) and (len(ret_data.asks) > depth_size):
                ret_data.asks = ret_data.asks[0:depth_size]

        return ret_data, req_cost, cost_manual




from huobi.client.trade import TradeClient
from huobi.constant import *


# cancel all the open orders under account
trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key)
result  = trade_client.cancel_open_orders(account_id=g_account_id)
result.print_object()



from huobi.model.margin.loan_ino import LoanInfo
from huobi.utils import default_parse_list_dict


class MarginLoanInfo:
    """
    The margin loan info.

    :member
        symbol: symbol like "btcusdt"
        currencies: loan info for currency in symbol
    """

    def __init__(self):
        self.symbol = ""
        self.currencies = list()

    @staticmethod
    def json_parse(json_data):
        retList = []
        for idx, item in enumerate(json_data):
            margin_loan_obj = MarginLoanInfo()
            margin_loan_obj.symbol = item.get("symbol", "")

            currencies_json = item.get("currencies")
            result_list = default_parse_list_dict(currencies_json, LoanInfo, [])

            margin_loan_obj.currencies = result_list

            retList.append(margin_loan_obj)

        return retList

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        if self.currencies and len(self.currencies):
            for currency_item in self.currencies:
                currency_item.print_object("\t")
                print()



from huobi.utils import *

from huobi.connection.subscribe_client import SubscribeClient
from huobi.model.account import *


class SubAccountUpdateV2Service:
    def __init__(self, params):
        self.params = params

    def subscribe(self, callback, error_handler, **kwargs):
        mode = self.params["mode"]

        def subscription(connection):
            connection.send(accounts_update_channel(mode))

        def parse(dict_data):
            account_change_event = AccountUpdateEvent()
            account_change_event.ch = dict_data.get("ch")
            data = dict_data.get("data", {})
            if data and len(data):
                account_change_event.data = default_parse_list_dict(data, AccountUpdate)

            return account_change_event

        SubscribeClient(**kwargs).execute_subscribe_v2(subscription,
                                                       parse,
                                                       callback,
                                                       error_handler,
                                                       is_trade=True)



from huobi.model.trade.order_update import OrderUpdate


class OrderUpdateEvent:
    """
    The order update received by subscription of order update.

    :member
        ch: The symbol you subscribed.
        data: The order detail.

    """

    def __init__(self):
        self.ch = ""
        self.data = OrderUpdate()

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.ch, format_data + "Topic")

        orderupdate = self.data
        orderupdate.print_object()



class DepthEntry:
    """
    An depth entry consisting of price and amount.

    :member
        price: The price of the depth.
        amount: The amount of the depth.
    """

    def __init__(self):
        self.price = 0.0
        self.amount = 0.0

    @staticmethod
    def json_parse(data_array):
        entry = DepthEntry()
        entry.price = data_array[0]
        entry.amount = data_array[1]
        return entry

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.price, format_data + "Price")
        PrintBasic.print_basic(self.amount, format_data + "Amount")



class WithdrawQuota:
    """
    Withdraw Quota info.

    :member
        chain: Block chain name.
        maxWithdrawAmt: Maximum withdraw amount in each request.
        withdrawQuotaPerDay: Maximum withdraw amount in a day
        remainWithdrawQuotaPerDay: Remaining withdraw quota in the day
        withdrawQuotaPerYear: Maximum withdraw amount in a year
        remainWithdrawQuotaPerYear: Remaining withdraw quota in the year
        withdrawQuotaTotal: Maximum withdraw amount in total
        remainWithdrawQuotaTotal: Remaining withdraw quota in total
    """
    def __init__(self):
        self.chain = ""
        self.maxWithdrawAmt = ""
        self.withdrawQuotaPerDay = ""
        self.remainWithdrawQuotaPerDay = ""
        self.withdrawQuotaPerYear = ""
        self.remainWithdrawQuotaPerYear = ""
        self.withdrawQuotaTotal = ""
        self.remainWithdrawQuotaTotal = ""



    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.chain, format_data + "Chain")
        PrintBasic.print_basic(self.maxWithdrawAmt, format_data + "maxWithdrawAmt")
        PrintBasic.print_basic(self.withdrawQuotaPerDay, format_data + "withdrawQuotaPerDay")
        PrintBasic.print_basic(self.remainWithdrawQuotaPerDay, format_data + "remainWithdrawQuotaPerDay")
        PrintBasic.print_basic(self.withdrawQuotaPerYear, format_data + "withdrawQuotaPerYear")
        PrintBasic.print_basic(self.remainWithdrawQuotaPerYear, format_data + "remainWithdrawQuotaPerYear")
        PrintBasic.print_basic(self.withdrawQuotaTotal, format_data + "withdrawQuotaTotal")
        PrintBasic.print_basic(self.remainWithdrawQuotaTotal, format_data + "remainWithdrawQuotaTotal")



from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant.system import HttpMethod
from huobi.model.etf import *

class GetEtfSwapListService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
      channel = "/etf/swap/list"

      def parse(dict_data):
        return EtfSwapList.json_parse_list(dict_data.get("data", []))

      return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)










from huobi.model.wallet.deposit import Deposit
from huobi.model.wallet.withdraw import Withdraw
from huobi.model.wallet.chain_deposit_address import ChainDepositAddress
from huobi.model.wallet.chain_withdraw_address import ChainWithdrawAddress
from huobi.model.wallet.withdraw_quota import WithdrawQuota
from huobi.model.wallet.deposit_history import DepositHistory
from huobi.model.wallet.deposit_history_item import DepositHistoryItem




class TradeDetail:
    """
    The trade information with price and amount etc.

    :member
        price: The trading price in quote currency.
        amount: The trading volume in base currency.
        tradeId: The unique trade id of this trade.
        timestamp: The UNIX formatted timestamp generated by server in UTC.
        direction: The direction of the taker trade: 'buy' or 'sell'.
    """

    def __init__(self):
        self.price = 0.0
        self.amount = 0.0
        self.tradeId = 0
        self.ts = 0
        self.direction = ""



    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.tradeId, format_data + "Trade Id")
        PrintBasic.print_basic(self.ts, format_data + "Trade Time")
        PrintBasic.print_basic(self.price, format_data + "Price")
        PrintBasic.print_basic(self.amount, format_data + "Amount")
        PrintBasic.print_basic(self.direction, format_data + "Direction")


class MarketDetail:
    """
    The summary of trading in the market for the last 24 hours

    :member
        id: response ID
        open: The opening price of last 24 hours.
        close: The last price of last 24 hours.
        amount: The aggregated trading volume in USDT.
        high: The high price of last 24 hours.
        low: The low price of last 24 hours.
        count: The number of completed trades.
        volume: The trading volume in base currency of last 24 hours.
        version: inner data
    """

    def __init__(self):
        self.id = 0
        self.open = 0.0
        self.close = 0.0
        self.amount = 0.0
        self.high = 0.0
        self.low = 0.0
        self.count = 0
        self.vol = 0.0
        self.version = 0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "ID")
        PrintBasic.print_basic(self.open, format_data + "Open")
        PrintBasic.print_basic(self.close, format_data + "Close")
        PrintBasic.print_basic(self.amount, format_data + "Amount")
        PrintBasic.print_basic(self.high, format_data + "High")
        PrintBasic.print_basic(self.low, format_data + "Low")
        PrintBasic.print_basic(self.count, format_data + "Count")
        PrintBasic.print_basic(self.vol, format_data + "Volume")
        # PrintBasic.print_basic(self.version, format_data + "Version")


from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant.system import HttpMethod
from huobi.model.account import *
from huobi.utils import *


class GetBalanceService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        account_id = self.params["account-id"]

        def get_channel():
            path = "/v1/account/accounts/{}/balance"
            return path.format(account_id)

        def parse(dict_data):
            data = dict_data.get("data", {})
            balance_list = data.get("list", [])
            return default_parse_list_dict(balance_list, Balance, [])

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, get_channel(), self.params, parse)

    def get_request(self, **kwargs):
        account_id = self.params["account-id"]

        def get_channel():
            path = "/v1/account/accounts/{}/balance"
            return path.format(account_id)

        def parse(dict_data):
            data = dict_data.get("data", {})
            balance_list = data.get("list", [])
            return default_parse_list_dict(balance_list, Balance, [])

        return RestApiSyncClient(**kwargs).create_request(HttpMethod.GET_SIGN, get_channel(), self.params, parse)



from huobi.client.account import AccountClient
from huobi.constant import *

account_client = AccountClient(api_key=g_api_key,
                               secret_key=g_secret_key)

point_transfer_result = account_client.post_point_transfer(from_uid=111859319, to_uid=124409916, group_id=0, amount=10)
point_transfer_result.print_object()




from huobi.client.wallet import WalletClient
from huobi.constant import *
from huobi.utils import LogInfo

wallet_client = WalletClient(api_key=g_api_key, secret_key=g_secret_key)
list_obj = wallet_client.get_sub_user_deposit_address(sub_uid=g_sub_uid, currency="btc")
LogInfo.output_list(list_obj)






import requests
from huobi.exception.huobi_api_exception import HuobiApiException
from huobi.utils.etf_result import etf_result_check
from huobi.utils import *
import time

from huobi.utils.print_mix_object import TypeCheck

session = requests.Session()

def check_response(dict_data):
    status = dict_data.get("status", None)
    code = dict_data.get("code", None)
    success = dict_data.get("success", None)
    if status and len(status):
        if TypeCheck.is_basic(status): # for normal case
            if status == "error":
                err_code = dict_data.get("err-code", 0)
                err_msg = dict_data.get("err-msg", "")
                raise HuobiApiException(HuobiApiException.EXEC_ERROR,
                                        "[Executing] " + str(err_code) + ": " + err_msg)
            elif status != "ok":
                raise HuobiApiException(HuobiApiException.RUNTIME_ERROR,
                                        "[Invoking] Response is not expected: " + status)
        elif TypeCheck.is_dict(status): # for https://status.huobigroup.com/api/v2/summary.json in example example/generic/get_system_status.py
            if dict_data.get("page") and dict_data.get("components"):
                pass
            else:
                raise HuobiApiException(HuobiApiException.EXEC_ERROR,
                                        "[Executing] System is in maintenances")
    elif code:
        code_int = int(code)
        if code_int != 200:
            err_code = dict_data.get("code", 0)
            err_msg = dict_data.get("message", "")
            raise HuobiApiException(HuobiApiException.EXEC_ERROR,
                                    "[Executing] " + str(err_code) + ": " + err_msg)
    elif success is not None:
        if bool(success) is False:
            err_code = etf_result_check(dict_data.get("code"))
            err_msg = dict_data.get("message", "")
            if err_code == "":
                raise HuobiApiException(HuobiApiException.EXEC_ERROR, "[Executing] " + err_msg)
            else:
                raise HuobiApiException(HuobiApiException.EXEC_ERROR, "[Executing] " + str(err_code) + ": " + err_msg)
    else:
        raise HuobiApiException(HuobiApiException.RUNTIME_ERROR, "[Invoking] Status cannot be found in response.")


def call_sync(request, is_checked=False):
    if request.method == "GET":
        # print("call_sync url : " , request.host + request.url)
        response = session.get(request.host + request.url, headers=request.header)
        if is_checked is True:
            return response.text
        dict_data = json.loads(response.text)#, encoding="utf-8")
        # print("call_sync  === recv data : ", dict_data)
        check_response(dict_data)
        return request.json_parser(dict_data)

    elif request.method == "POST":
        response = session.post(request.host + request.url, data=json.dumps(request.post_body), headers=request.header)
        dict_data = json.loads(response.text, encoding="utf-8")
        # print("call_sync  === recv data : ", dict_data)
        check_response(dict_data)
        return request.json_parser(dict_data)

def call_sync_perforence_test(request, is_checked=False):
    if request.method == "GET":
        inner_start_time = time.time()
        # print("call_sync_perforence_test url : ", request.host + request.url)
        response = session.get(request.host + request.url, headers=request.header)
        #print("call_sync_perforence_test data :", response.text)
        inner_end_time = time.time()
        cost_manual = round(inner_end_time - inner_start_time, 6)
        req_cost = response.elapsed.total_seconds()
        if is_checked is True:
            return response.text
        dict_data = json.loads(response.text, encoding="utf-8")
        # print("call_sync  === recv data : ", dict_data)
        check_response(dict_data)
        return request.json_parser(dict_data), req_cost, cost_manual

    elif request.method == "POST":
        inner_start_time = time.time()
        response = session.post(request.host + request.url, data=json.dumps(request.post_body), headers=request.header)
        inner_end_time = time.time()
        cost_manual = round(inner_end_time - inner_start_time, 6)
        req_cost = response.elapsed.total_seconds()
        dict_data = json.loads(response.text, encoding="utf-8")
        # print("call_sync  === recv data : ", dict_data)
        check_response(dict_data)
        return request.json_parser(dict_data), req_cost, cost_manual



from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant.system import HttpMethod
from huobi.utils.json_parser import default_parse_data_as_long


class PostSubaccountTransferService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/subuser/transfer"

        def parse(dict_data):
            return default_parse_data_as_long(dict_data, None)


        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)









from huobi.constant import *


class OrderUpdate:
    """
    The detail order information.

    :member
        orderId: The order id.
        tradePrice: trade price
        tradeVolume: trade volume
        tradeId: Id record for trade
        tradeTime: trade timestamp (ms)
        aggressor: true (taker), false (maker)
        remainAmt: Remaining amount (for buy-market order it's remaining value)
        orderStatus: Order status, valid value: partial-filled, filled
        clientOrderId: Client order ID (if any)
        eventType: Event type, valid value: trade
        symbol: The symbol, like "btcusdt".
        type: The order type, possible values are: buy-market, sell-market, buy-limit, sell-limit, buy-ioc, sell-ioc, buy-limit-maker, sell-limit-maker, buy-limit-fok, sell-limit-fok.
    """

    def __init__(self):
        self.orderId = 0
        self.tradePrice = ""
        self.tradeVolume = ""
        self.tradeId = 0
        self.tradeTime = 0
        self.aggressor = False
        self.remainAmt = ""
        self.orderStatus = OrderState.INVALID
        self.clientOrderId = ""
        self.eventType = ""
        self.symbol = ""
        self.type = OrderType.INVALID
        self.accountId = 0


    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.orderId, format_data + "Order Id")
        PrintBasic.print_basic(self.tradePrice, format_data + "Trade Price")
        PrintBasic.print_basic(self.tradeVolume, format_data + "Trade Volume")
        PrintBasic.print_basic(self.tradeId, format_data + "Trade Id")
        PrintBasic.print_basic(self.tradeTime, format_data + "Trade Timestamp")
        PrintBasic.print_basic(self.aggressor, format_data + "is Taker")
        PrintBasic.print_basic(self.orderStatus, format_data + "Order State")
        PrintBasic.print_basic(self.clientOrderId, format_data + "Client Order Id")
        PrintBasic.print_basic(self.eventType, format_data + "Event Type")
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        PrintBasic.print_basic(self.type, format_data + "Order Type")
        PrintBasic.print_basic(self.accountId, format_data + "Account Id")







class LastTradeAndBestQuote:
    """
    The last trade and best bid/ask.

    :member
        last_trade_price: The last trade price.
        last_trade_amount: The last trade amount.
        ask_price: The best ask price.
        ask_amount: The best ask amount.
        bid_price: The best bid price.
        bid_amount: The best bid amount.

    """

    def __init__(self):
        self.last_trade_price = 0.0
        self.last_trade_amount = 0.0
        self.ask_price = 0.0
        self.ask_amount = 0.0
        self.bid_price = 0.0
        self.bid_amount = 0.0




from huobi.client.market import MarketClient
from huobi.model.market import *


def callback(obj_event: 'MarketDetailEvent'):
    obj_event.print_object()
    print()


market_client = MarketClient()
market_client.sub_market_detail("btcusdt", callback)


from huobi.client.etf import EtfClient
from huobi.constant import *
from huobi.utils import *


etf_client = EtfClient(api_key=g_api_key, secret_key=g_secret_key)
etf_list = etf_client.get_etf_swap_list(etf_name="hb10", offset=0, size=20)
LogInfo.output_list(etf_list)



















from huobi.client.subuser import SubuserClient
from huobi.constant import *
from huobi.utils import *

subuser_client = SubuserClient(api_key=g_api_key, secret_key=g_secret_key)
uid = subuser_client.get_uid()
LogInfo.output(uid)



from huobi.constant import *
from huobi.utils import default_parse_list_dict


class Balance:
    """
    The balance of specified account.

    :member
        currency: The currency of this balance.
        balance_type: The balance type, trade or frozen.
        balance: The balance in the main currency unit.

    """

    def __init__(self):
        self.currency = ""
        self.type = AccountBalanceUpdateType.INVALID
        self.balance = 0.0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.type, format_data + "Balance Type")
        PrintBasic.print_basic(self.balance, format_data + "Balance")


from huobi.client.trade import TradeClient
from huobi.constant import *
from huobi.utils import *

symbol_test_list = ["eosusdt"]
trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key)
for symbol_test in symbol_test_list:
    list_obj = trade_client.get_history_orders(symbol=symbol_test, start_time=None, end_time=None, size=20, direct=None)
    LogInfo.output_list(list_obj)


















class AccountHistory:
    """
    The account information for spot account, margin account etc.

    :member
        account_id: Account Id.
        currency: Currency name
        transact_amt: Amount change (positive value if income, negative value if outcome)
        transact-type: Amount change type
        avail_balance: Available balance
        acct_balance: Account balance
        transact_time: Transaction time (database time)
        record_id: Unique record ID in the database

    """

    def __init__(self):
        self.account_id = 0
        self.currency = ""
        self.transact_amt = ""
        self.transact_type = ""
        self.avail_balance = ""
        self.acct_balance = ""
        self.transact_time = 0
        self.record_id = 0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.account_id, format_data + "Account Id")
        PrintBasic.print_basic(self.currency, format_data + "Currency")
        PrintBasic.print_basic(self.transact_amt, format_data + "Transact Amount")
        PrintBasic.print_basic(self.transact_type, format_data + "Transact Type")
        PrintBasic.print_basic(self.avail_balance, format_data + "Avail Balance")
        PrintBasic.print_basic(self.acct_balance, format_data + "Account Balance")
        PrintBasic.print_basic(self.transact_time, format_data + "Transact Time")
        PrintBasic.print_basic(self.record_id, format_data + "Record Id")



import logging

from huobi.connection.impl.websocket_manage import WebsocketManage
from huobi.connection.impl.websocket_request import WebsocketRequest
from huobi.constant.system import WebSocketDefine


class WebSocketReqClient(object):

    def __init__(self, **kwargs):
        """
        Create the subscription client to subscribe the update from server.

        :param kwargs: The option of subscription connection.
            api_key: The public key applied from Huobi.
            secret_key: The private key applied from Huobi.
            url: Set the URI for subscription.
            init_log: to init logger
        """

        self.__api_key = kwargs.get("api_key", None)
        self.__secret_key = kwargs.get("secret_key", None)
        self.__uri = kwargs.get("url", WebSocketDefine.Uri)
        self.__init_log = kwargs.get("init_log", None)
        if self.__init_log and self.__init_log:
            logger = logging.getLogger("huobi-client")
            logger.setLevel(level=logging.INFO)
            handler = logging.StreamHandler()
            handler.setFormatter(logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s'))
            logger.addHandler(handler)

    def __create_websocket_manage(self, request):
        manager = WebsocketManage(self.__api_key, self.__secret_key, self.__uri, request)
        manager.connect()

    def create_request(self, subscription_handler, parse, callback, error_handler, is_trade=False, is_mbp_feed=False):
        request = WebsocketRequest()
        request.subscription_handler = subscription_handler
        request.is_trading = is_trade
        request.is_mbp_feed = is_mbp_feed
        request.auto_close = True  # for websocket request, auto close the connection after request.
        request.json_parser = parse
        request.update_callback = callback
        request.error_handler = error_handler
        return request

    def execute_subscribe_v1(self, subscription_handler, parse, callback, error_handler, is_trade=False):
        request = self.create_request(subscription_handler, parse, callback, error_handler, is_trade)
        self.__create_websocket_manage(request)

    def execute_subscribe_mbp(self, subscription_handler, parse, callback, error_handler, is_trade=False,
                              is_mbp_feed=True):
        request = self.create_request(subscription_handler, parse, callback, error_handler, is_trade, is_mbp_feed)
        self.__create_websocket_manage(request)



from huobi.model.market import PriceDepthBbo


class PriceDepthBboEvent:
    """
    The price depth received by subscription of price depth.

    :member
        symbol: The symbol you subscribed.
        timestamp: The UNIX formatted timestamp generated by server in UTC.
        data: The price depth.

    """

    def __init__(self):
        self.ts = 0
        self.ch = ""
        self.tick = PriceDepthBbo()


    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.ts, format_data + "Time")
        PrintBasic.print_basic(self.ch, format_data + "Channel")
        self.tick.print_object(format_data)



from huobi.client.market import MarketClient
from huobi.constant import *



def callback(price_depth_event: 'PriceDepthEvent'):
    price_depth_event.print_object()


def error(e: 'HuobiApiException'):
    print(e.error_code + e.error_message)


market_client = MarketClient()
market_client.sub_pricedepth("btcusdt", DepthStep.STEP0, callback, error)
market_client.sub_pricedepth("eosusdt", DepthStep.STEP0, callback, error)
market_client.sub_pricedepth("ethusdt", DepthStep.STEP0, callback, error)



class BatchCancelResult:
    """
    The result of batch cancel operation.

    :member
        success_count: The number of cancel request sent successfully.
        failed_count: The number of cancel request failed.

    """

    def __init__(self):
        self.success = []
        self.failed = []

    def print_object(self, format_data=""):
        print("Success Order Counts", len(self.success), " Success Order Ids : ", self.success)
        print("Fail Order Counts", len(self.failed), " Fail Order Ids : ", self.failed)




from huobi.model.market.depth_entry import DepthEntry

class Mbp:
    """
    Increasement of price depth information.

    :member
        seqNum: current seqNum.
        prevSeqNum: previous seqNum.
        bids: The list of the bid depth. The content is DepthEntry class.
        asks: The list of the ask depth. The content is DepthEntry class.

    """
    def __init__(self):
        self.seqNum = 0
        self.prevSeqNum = 0
        self.bids = list()
        self.asks = list()

    @staticmethod
    def json_parse(json_data):
        mbp = Mbp()
        bid_list = list()
        mbp.seqNum = json_data.get("seqNum", 0)
        mbp.prevSeqNum = json_data.get("prevSeqNum", 0)  # prevSeqNum only for increased subscribe, request doesn't have this value
        for item in json_data.get("bids", []):
            depth_entry = DepthEntry()
            depth_entry.price = item[0]
            depth_entry.amount = item[1]
            bid_list.append(depth_entry)
        ask_list = list()
        for item in json_data.get("asks", []):
            depth_entry = DepthEntry()
            depth_entry.price = item[0]
            depth_entry.amount = item[1]
            ask_list.append(depth_entry)
        mbp.bids = bid_list
        mbp.asks = ask_list

        return mbp



    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.seqNum, format_data + "seqNum")
        PrintBasic.print_basic(self.prevSeqNum, format_data + "prevSeqNum")
        for entry in self.bids:
            PrintBasic.print_basic(str(entry.price) + ", amount: " + str(entry.amount), format_data + "Bids price: ")

        for entry in self.asks:
            PrintBasic.print_basic(str(entry.price) + ", amount: " + str(entry.amount), format_data + "Asks price: ")



from huobi.constant import *
from huobi.utils.json_parser import fill_obj


class Order:
    """
    The detail order information.

    :member
        amount: The amount of base currency in this order.
        price: The limit price of limit order.
        created_timestamp: The UNIX formatted timestamp in UTC when the order was created.
        canceled_timestamp: The UNIX formatted timestamp in UTC when the order was canceled, if not canceled then has value of 0.
        finished_timestamp: The UNIX formatted timestamp in UTC when the order was changed to a final state. This is not the time the order is matched.
        order_id: The order id.
        symbol: The symbol, like "btcusdt".
        order_type: The order type, possible values are: buy-market, sell-market, buy-limit, sell-limit, buy-ioc, sell-ioc, buy-limit-maker, sell-limit-maker, buy-limit-fok, sell-limit-fok, buy-stop-limit-fok, sell-stop-limit-fok.
        filled_amount: The amount which has been filled.
        filled_cash_amount: The filled total in quote currency.
        filled_fees: The transaction fee paid so far.
        source: The source where the order was triggered, possible values: sys, web, api, app.
        state: The order state: submitted, partial-filled, cancelling, filled, canceled.
        stop_price : stop price used for buy-stop-limit，sell-stop-limit
        operator : only [gte] and [lte] to trigger buy-stop-limit，sell-stop-limit
    """

    def __init__(self):
        self.id = 0
        self.symbol = ""
        self.account_id = 0
        self.amount = 0.0
        self.price = 0.0
        self.created_at = 0
        self.canceled_at = 0
        self.finished_at = 0
        self.type = OrderType.INVALID
        self.filled_amount = 0.0
        self.filled_cash_amount = 0.0
        self.filled_fees = 0.0
        self.source = OrderSource.INVALID
        self.state = OrderState.INVALID
        self.client_order_id = ""
        self.stop_price = ""
        self.next_time = 0
        self.operator=""

    @staticmethod
    def json_parse(json_data):
        order = fill_obj(json_data, Order)
        order.filled_amount = json_data.get("filled-amount", json_data.get("field-amount", 0))
        order.filled_cash_amount = json_data.get("filled-cash-amount", json_data.get("field-cash-amount", 0))
        order.filled_fees = json_data.get("filled-fees", json_data.get("field-fees", 0))
        return order

    @staticmethod
    def json_parse_list(json_data):
        if json_data and len(json_data):
            order_list = list()
            for idx, row in enumerate(json_data):
                order_item = Order.json_parse(row)
                order_list.append(order_item)
            return order_list

        return list()

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "Order Id")
        PrintBasic.print_basic(self.symbol, format_data + "Symbol")
        PrintBasic.print_basic(self.price, format_data + "Price")
        PrintBasic.print_basic(self.amount, format_data + "Amount")
        PrintBasic.print_basic(self.created_at, format_data + "Create Time")
        PrintBasic.print_basic(self.canceled_at, format_data + "Cancel Time")
        PrintBasic.print_basic(self.finished_at, format_data + "Finish Time")
        PrintBasic.print_basic(self.type, format_data + "Order Type")
        PrintBasic.print_basic(self.filled_amount, format_data + "Filled Amount")
        PrintBasic.print_basic(self.filled_cash_amount, format_data + "Filled Cash Amount")
        PrintBasic.print_basic(self.filled_fees, format_data + "Filled Fees")
        #PrintBasic.print_basic(self.account_type, format_data + "Account Type")
        PrintBasic.print_basic(self.source, format_data + "Order Source")
        PrintBasic.print_basic(self.state, format_data + "Order State")
        PrintBasic.print_basic(self.client_order_id, format_data + "Client Order Id")
        PrintBasic.print_basic(self.stop_price, format_data + "Stop Price")
        PrintBasic.print_basic(self.operator, format_data + "Operator")
        PrintBasic.print_basic(self.next_time, format_data + "Next Time")






from huobi.connection.websocket_req_client import *
from huobi.model.trade import *
from huobi.utils import *


class ReqOrderDetailService:
    def __init__(self, params):
        self.params = params

    def subscribe(self, callback, error_handler, **kwargs):
        order_id = self.params["order-id"]
        client_req_id = self.params["cid"]

        def subscription(connection):
            connection.send(request_order_detail_channel(order_id, client_req_id))

        def parse(dict_data):
            order_update_event = default_parse(dict_data, OrderDetailReq, OrderListItem)

            return order_update_event

        WebSocketReqClient(**kwargs).execute_subscribe_v1(subscription,
                                            parse,
                                            callback,
                                            error_handler,
                                            is_trade=True)










from huobi.constant import *


class GeneralRepayLoanResult:
    """
    The margin order information.

    :member
        id: Inner id.
        type: The account type.
        state: The account state.
        symbol: The symbol, like "btcusdt".
        fl_price: The trigger price.
        fl_type: The trigger type.
        risk_rate: The risk rate.
        list:Balance Object list
    """

    def __init__(self):
        self.repayId = 0
        self.repayTime = 0

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.repayId, format_data + "repayId")
        PrintBasic.print_basic(self.repayTime, format_data + "repayTime")

        print()






from huobi.client.market import MarketClient
from huobi.utils import *

market_client = MarketClient()
list_obj = market_client.get_market_trade(symbol="eosusdt")
LogInfo.output_list(list_obj)















from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant import *
from huobi.model.trade import *
from huobi.utils import *
from huobi.utils.json_parser import default_parse_data_as_long


class PostCreateWithdrawService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/dw/withdraw/api/create"

        def parse(dict_data):
            # return dict_data.get("data", 0)
            return default_parse_data_as_long(dict_data, None)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)









from huobi.client.trade import TradeClient
from huobi.constant import *
from huobi.utils import *


trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key)
symbol_test = "eosusdt"

i = 0
n = 3
order_id_list = []
while(i<n):
    order_id = trade_client.create_order(symbol=symbol_test, account_id=g_account_id, order_type=OrderType.BUY_LIMIT, source=OrderSource.API, amount=18.0, price=0.292)
    LogInfo.output("created order id : {id}".format(id=order_id))
    order_id_list.append(order_id)
    i = i+1

result = trade_client.cancel_orders(symbol_test, order_id_list)
result.print_object()





from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant.system import HttpMethod


class PostSubuserApikeyDeletionService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/sub-user/api-key-deletion"

        # {'code': 200, 'data': None, 'ok': True}
        def parse(dict_data):
            return dict_data

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)



from huobi.constant import *


class EtfSwapConfig:
    """
    The basic information of ETF creation and redemption, as well as ETF constituents, including max
    amount of creation, min amount of creation, max amount of redemption, min amount of redemption,
    creation fee rate, redemption fee rate, eft create/redeem status.

    :member
        purchase_max_amount: The max creation amounts per request.
        purchase_min_amount: The minimum creation amounts per request.
        redemption_max_amount: The max redemption amounts per request.
        redemption_min_amount: The minimum redemption amounts per request.
        purchase_fee_rate: The creation fee rate.
        redemption_fee_rate: The redemption fee rate.
        status: The status of the ETF.
        unit_price_list: ETF constitution in format of amount and currency.

    """
    def __init__(self):
        self.etf_name = ""
        self.etf_status = EtfStatus.INVALID
        self.purchase_fee_rate = 0.0
        self.purchase_max_amount = 0
        self.purchase_min_amount = 0
        self.redemption_fee_rate = 0.0
        self.redemption_max_amount = 0
        self.redemption_min_amount = 0
        self.unit_price = list()

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.etf_name, format_data + "ETF Name")
        PrintBasic.print_basic(self.etf_status, format_data + "ETF Status")

        PrintBasic.print_basic(self.purchase_fee_rate, format_data + "Purchase Fee Rate")
        PrintBasic.print_basic(self.purchase_max_amount, format_data + "Purchase Max Amount")
        PrintBasic.print_basic(self.purchase_min_amount, format_data + "Purchase Min Amount")

        PrintBasic.print_basic(self.redemption_fee_rate, format_data + "Redemption Fee Rate")
        PrintBasic.print_basic(self.redemption_max_amount, format_data + "Redemption Max Amount")
        PrintBasic.print_basic(self.redemption_min_amount, format_data + "Redemption Min Amount")
        print()
        if len(self.unit_price):
            for row in self.unit_price:
                row.print_object(format_data)
                print()




from huobi.client.account import AccountClient
from huobi.constant import *

# get accounts
account_client = AccountClient(api_key=g_api_key,
                              secret_key=g_secret_key)

ret = account_client.post_sub_uid_management(sub_uid=g_sub_uid, action=SubUidAction.LOCK)
ret.print_object()

ret = account_client.post_sub_uid_management(sub_uid=g_sub_uid, action=SubUidAction.UNLOCK)
ret.print_object()


from huobi.client.market import MarketClient



def callback(trade_req: 'TradeDetailReq'):
    print("---- trade_event:  ----")
    trade_req.print_object()
    print()



market_client = MarketClient()
market_client.req_trade_detail("btcusdt,eosusdt", callback)





#!/usr/bin/env python3
from setuptools import setup

setup(
    name="huobi_custom",
    version="2.0.2",
    packages=['huobi',
              'huobi.exception', 'huobi.constant',
              'huobi.utils',
              'huobi.client',
              'huobi.service', 'huobi.service.account', 'huobi.service.margin', 'huobi.service.market',
              'huobi.service.trade', 'huobi.service.wallet', 'huobi.service.generic', 'huobi.service.etf',
              'huobi.service.subuser', 'huobi.service.algo',
              'huobi.model', 'huobi.model.account', 'huobi.model.margin', 'huobi.model.market', 'huobi.model.trade',
              'huobi.model.wallet', 'huobi.model.generic', 'huobi.model.etf', 'huobi.model.subuser', 'huobi.model.algo',
              'huobi.connection', 'huobi.connection.impl', "performance", "tests"
              ],
    install_requires=['requests', 'apscheduler', 'websocket-client', 'urllib3']
)



from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant.system import HttpMethod


class PostCrossMarginLoanOrderRepayService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/cross-margin/orders/{order_id}/repay".format(order_id=self.params.get("order-id"))

        def parse(dict_data):
            return dict_data.get("status", None)

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.POST_SIGN, channel, self.params, parse)












class BatchCancelCount:
    """
    The result of batch cancel operation.

    :member
        success_count: The number of cancel request sent successfully.
        failed_count: The number of cancel request failed.
        next_id:next open order id
    """

    def __init__(self):
        self.success_count = 0
        self.failed_count = 0
        self.next_id = -1

    def print_object(self, format_data=""):
        from huobi.utils import PrintBasic
        PrintBasic.print_basic(self.success_count, format_data + "Success Count")
        PrintBasic.print_basic(self.failed_count, format_data + "Failed Count")
        PrintBasic.print_basic(self.next_id, format_data + "Next Open Order ID")



class MarketDetailMerged:
    """
    The best bid/ask consisting of price and amount.

    :member
        timestamp: The Unix formatted timestamp in UTC.
        bid_price: The best bid price.
        bid_amount: The best bid amount.
        ask_price: The best ask price.
        ask_amount: The best ask amount.

    """

    def __init__(self):
        self.amount = 0
        self.open = 0.0
        self.close = 0.0
        self.high = 0.0
        self.id = 0
        self.count = 0.0
        self.low = 0.0
        self.version = 0
        self.ask = []
        self.vol = 0.0
        self.bid = []

    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        # PrintBasic.print_basic(self.version, format_data + "Version")
        PrintBasic.print_basic(self.amount, format_data + "Amount")
        PrintBasic.print_basic(self.count, format_data + "Count")
        PrintBasic.print_basic(self.vol, format_data + "Volume")

        PrintBasic.print_basic(self.open, format_data + "Open")
        PrintBasic.print_basic(self.close, format_data + "Close")
        PrintBasic.print_basic(self.high, format_data + "High")
        PrintBasic.print_basic(self.low, format_data + "Low")

        print("Ask", self.ask)
        print("Bid", self.bid)



from huobi.client.wallet import WalletClient
from huobi.constant import *


wallet_client = WalletClient(api_key=g_api_key, secret_key=g_secret_key)
deposit_history = wallet_client.get_sub_user_deposit_history(sub_uid=g_sub_uid)
deposit_history.print_object()







from huobi.client.margin import MarginClient
from huobi.constant import *
from huobi.utils import *

margin_client = MarginClient(api_key=g_api_key, secret_key=g_secret_key)
transfer_id = margin_client.post_transfer_in_margin(symbol="eosusdt", currency="usdt", amount=20)
LogInfo.output("transfer id : {id}".format(id=transfer_id))




import logging

from huobi.connection.impl.restapi_invoker import call_sync, call_sync_perforence_test
from huobi.connection.impl.restapi_request import RestApiRequest
from huobi.constant import *
from huobi.utils import *

from huobi.exception.huobi_api_exception import HuobiApiException



class RestApiSyncClient(object):

    def __init__(self, **kwargs):
        """
        Create the request client instance.
        :param kwargs: The option of request connection.
            api_key: The public key applied from Huobi.
            secret_key: The private key applied from Huobi.
            url: The URL name like "https://api.huobi.pro".
            performance_test: for performance test
            init_log: to init logger
        """
        self.__api_key = kwargs.get("api_key", None)
        self.__secret_key = kwargs.get("secret_key", None)
        self.__server_url = kwargs.get("url", get_default_server_url(None))
        self.__init_log = kwargs.get("init_log", None)
        self.__performance_test = kwargs.get("performance_test", None)
        if self.__init_log and self.__init_log:
            logger = logging.getLogger("huobi-client")
            logger.setLevel(level=logging.INFO)
            handler = logging.StreamHandler()
            handler.setFormatter(logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s'))
            logger.addHandler(handler)

    def __create_request_by_get(self, url, builder):
        request = RestApiRequest()
        request.method = "GET"
        request.host = self.__server_url
        request.header.update({'Content-Type': 'application/json'})
        request.url = url + builder.build_url()
        return request

    def __create_request_by_post_with_signature(self, url, builder):
        request = RestApiRequest()
        request.method = "POST"
        request.host = self.__server_url
        create_signature(self.__api_key, self.__secret_key, request.method, request.host + url, builder)
        request.header.update({'Content-Type': 'application/json'})
        if (len(builder.post_list)):  # specify for case : /v1/order/batch-orders
            request.post_body = builder.post_list
        else:
            request.post_body = builder.post_map
        request.url = url + builder.build_url()
        return request

    def __create_request_by_get_with_signature(self, url, builder):
        request = RestApiRequest()
        request.method = "GET"
        request.host = self.__server_url
        create_signature(self.__api_key, self.__secret_key, request.method, request.host + url, builder)
        request.header.update({"Content-Type": "application/x-www-form-urlencoded"})
        request.url = url + builder.build_url()
        return request

    def create_request(self, method, url, params, parse):
        builder = UrlParamsBuilder()
        if params and len(params):
            if method in [HttpMethod.GET, HttpMethod.GET_SIGN]:
                for key, value in params.items():
                    builder.put_url(key, value)
            elif method in [HttpMethod.POST, HttpMethod.POST_SIGN]:
                for key, value in params.items():
                    builder.put_post(key, value)
            else:
                raise HuobiApiException(HuobiApiException.EXEC_ERROR,
                                        "[error] undefined HTTP method")

        if method == HttpMethod.GET:
            request = self.__create_request_by_get(url, builder)
        elif method == HttpMethod.GET_SIGN:
            request = self.__create_request_by_get_with_signature(url, builder)
        elif method == HttpMethod.POST_SIGN:
            request = self.__create_request_by_post_with_signature(url, builder)
        elif method == HttpMethod.POST:
            request = self.__create_request_by_post_with_signature(url, builder)
        else:
            raise HuobiApiException(HuobiApiException.INPUT_ERROR, "[Input] " + method + "  is invalid http method")

        request.json_parser = parse

        return request

    """
    for post batch operation, such as batch create orders[ /v1/order/batch-orders ]
    """
    def create_request_post_batch(self, method, url, params, parse):
        builder = UrlParamsBuilder()
        if params and len(params):
            if method in [HttpMethod.POST, HttpMethod.POST_SIGN]:
                if isinstance(params, list):
                    builder.post_list = params
            else:
                raise HuobiApiException(HuobiApiException.EXEC_ERROR,
                                        "[error] undefined HTTP method")

        request = self.__create_request_by_post_with_signature(url, builder)
        request.json_parser = parse

        return request

    def request_process(self, method, url, params, parse):
        if self.__performance_test is not None and self.__performance_test is True:
            return self.request_process_performance(method, url, params, parse)
        else:
            return self.request_process_product(method, url, params, parse)

    def request_process_product(self, method, url, params, parse):
        request = self.create_request(method, url, params, parse)
        if request:
            return call_sync(request)

        return None

    def request_process_performance(self, method, url, params, parse):
        request = self.create_request(method, url, params, parse)
        if request:
            return call_sync_perforence_test(request)

        return None, 0, 0

    """
    for post batch operation, such as batch create orders[ /v1/order/batch-orders ]
    """
    def request_process_post_batch(self, method, url, params, parse):
        if self.__performance_test is not None and self.__performance_test is True:
            return self.request_process_post_batch_performance(method, url, params, parse)
        else:
            return self.request_process_post_batch_product(method, url, params, parse)

    def request_process_post_batch_product(self, method, url, params, parse):
        request = self.create_request_post_batch(method, url, params, parse)
        if request:
            return call_sync(request)

        return None

    def request_process_post_batch_performance(self, method, url, params, parse):
        request = self.create_request_post_batch(method, url, params, parse)
        if request:
            return call_sync_perforence_test(request)

        return None, 0, 0




from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant.system import HttpMethod
from huobi.model.margin import *
from huobi.utils import *



class GetMarginLoanInfoService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v1/margin/loan-info"

        def parse(dict_data):
            return MarginLoanInfo.json_parse(dict_data.get("data", []))

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)









from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant import *
from huobi.model.wallet import *
from huobi.utils import *


class GetSubUserDepositHistoryService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/v2/sub-user/query-deposit"

        def parse(dict_data):
            deposit_history = DepositHistory()
            deposit_history.nextId = dict_data.get("nextId", 0)
            json_data_list = dict_data.get("data", [])
            deposit_history_item_list = default_parse_list_dict(json_data_list, DepositHistoryItem)
            deposit_history.data = deposit_history_item_list
            return deposit_history

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET_SIGN, channel, self.params, parse)






from huobi.connection.restapi_sync_client import RestApiSyncClient
from huobi.constant.system import HttpMethod
from huobi.model.market import *
from huobi.utils import *


class GetMarketTradeService:

    def __init__(self, params):
        self.params = params

    def request(self, **kwargs):
        channel = "/market/trade"

        def parse(dict_data):
            tick = dict_data.get("tick", {})
            data = tick.get("data", [])
            return default_parse_list_dict(data, Trade, [])

        return RestApiSyncClient(**kwargs).request_process(HttpMethod.GET, channel, self.params, parse)










from huobi.client.trade import TradeClient
from huobi.constant import *


def callback(upd_event: 'OrderUpdateEvent'):
    print("---- order update : ----")
    upd_event.print_object()
    print()


trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key, init_log=True)
trade_client.sub_order_update("eosusdt", callback)



from huobi.client.generic import GenericClient
from huobi.model.generic import ExchangeInfo


class GenericClientPerformance(GenericClient):

    def __init__(self, **kwargs):
        """
        Create the request client instance.
        :param kwargs: The option of request connection.
            api_key: The public key applied from Huobi.
            secret_key: The private key applied from Huobi.
            url: The URL name like "https://api.huobi.pro".
            init_log: to init logger
        """

        self.__kwargs = kwargs
        self.__kwargs["performance_test"] = True
        super(GenericClientPerformance, self).__init__(**self.__kwargs)

    def get_exchange_info(self) -> ExchangeInfo:
        """
        Get all the trading assets and currencies supported in huobi.
        The information of trading instrument, including base currency, quote precision, etc.

        :return: The information of trading instrument and currencies.
        """

        ret = ExchangeInfo()
        ret.symbol_list, req_cost_1, cost_manual_1  = self.get_exchange_symbols()
        ret.currencies, req_cost_2, cost_manual_2 = self.get_exchange_currencies()
        return ret, req_cost_1 + req_cost_2, cost_manual_1 + cost_manual_2


if __name__ == "__main__":
    generic_client = GenericClientPerformance()
    list_obj, req_cost, cost_manual = generic_client.get_exchange_info()

    print(len(list_obj.symbol_list), req_cost, cost_manual)

    # generic_client = GenericClient(performance_test=True)
    # list_obj, req_cost, cost_manual = generic_client.get_exchange_symbols()
    #
    # print(len(list_obj), req_cost, cost_manual)




from huobi.client.subuser import SubuserClient
from huobi.constant import *

subuser_client = SubuserClient(api_key=g_api_key, secret_key=g_secret_key)

otp_token = '746316'
sub_uid = 122946475
note = "huobi_subuser"
permission = 'readOnly'
# ip_addresses = ''

result = subuser_client.post_subuser_apikey_generate(otp_token, sub_uid, note, permission)
result.print_object()



class BatchCreateOrder:
    """
    batch create order result

    :member
        order_id: The transfer id.
        client_order_id: The crypto currency to deposit.
        err_code: The on-chain transaction hash.
        err_msg: The number of crypto asset transferred in its minimum unit.

    """
    def __init__(self):
        self.order_id = 0
        self.client_order_id = ""
        self.err_code = ""
        self.err_msg = ""

    def print_object(self, format_data=""):
        from huobi.utils import PrintBasic
        PrintBasic.print_basic(self.order_id, format_data + "Order Id")
        PrintBasic.print_basic(self.client_order_id, format_data + "Client Order Id")
        PrintBasic.print_basic(self.err_code, format_data + "Error Code")
        PrintBasic.print_basic(self.err_msg, format_data + "Error Message")



from huobi.model.margin.loan_ino import LoanInfo

class CrossMarginLoanInfo(LoanInfo):
    def __init__(self):
        LoanInfo.__init__(self)

    def print_object(self, format_data=""):
        LoanInfo.print_object(self)



from huobi.client.generic import GenericClient
from huobi.utils import *


generic_client = GenericClient()
list_obj = generic_client.get_exchange_currencies()
LogInfo.output("---- Supported currency ----")
for currency in list_obj:
    LogInfo.output(currency)


import time

from huobi.model.market import *
from huobi.utils import *
from huobi.connection.subscribe_client import SubscribeClient


class SubMbpIncreaseService:
    def __init__(self, params):
        self.params = params

    def subscribe(self, callback, error_handler, **kwargs):
        symbol_list = self.params["symbol_list"]
        level = self.params["levels"]

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(mbp_increase_channel(symbol, level))
                time.sleep(0.01)

        def parse(dict_data):
            return MbpIncreaseEvent.json_parse(dict_data)

        SubscribeClient(**kwargs).execute_subscribe_mbp(subscription,
                                            parse,
                                            callback,
                                            error_handler)






from huobi.client.etf import EtfClient
from huobi.client.generic import GenericClient
import time

from huobi.client.margin import MarginClient
from huobi.client.trade import TradeClient
from huobi.client.wallet import WalletClient
from huobi.constant import *
from performance.account_performance import AccountClientPerformance
from performance.market_performance import MarketClientPerformance


class RunStatus:
    SUCCESS = "OK"
    FAILED = "Fail"

ROUND_SIZE = 3
TRANSFER_TRX_MIN_AMOUNT = 100

time_cost_detail_list = []
count_offset = 0
from huobi.constant.test import g_api_key, g_secret_key

# prepare list
withdraw_address = "TRec1vonZcbcXHs5TZLuH2WTa8ejXvnTm8"
loan_amount = 100
loan_currency = "usdt"
loan_symbol = "eosusdt"
trade_symbol = "eosusdt"



class TimeCost:
    sdk_api_start_time = 0.0  # SDK call start time
    server_req_cost = 0.0 # time cost from response.elapsed.total_seconds(), cost is from sending request to receive response
    server_api_cost = 0.0   # manually statistics time before/after requests.get  (server_api_cost >= server_req_cost)
    function_name = ""
    run_status = ""

    def __init__(self, function_name=""):
        self.sdk_api_start_time = round(time.time(), ROUND_SIZE + 1)
        self.function_name = function_name

    def add_record(self):
        sdk_api_end_time = round(time.time(), ROUND_SIZE + 1)
        sdk_api_cost = sdk_api_end_time - self.sdk_api_start_time
        sdk_cost_req = sdk_api_cost - self.server_req_cost
        sdk_cost_manual = sdk_api_cost - self.server_api_cost

        row_dict = {
            "sdk_api_cost" : round(sdk_api_cost, ROUND_SIZE),
            "server_api_cost" : round(self.server_api_cost, ROUND_SIZE + 1),
            "server_req_cost" : round(self.server_req_cost, ROUND_SIZE + 1),
            "sdk_api_delay" : round(sdk_cost_manual, ROUND_SIZE),
            "sdk_req_delay" : round(sdk_cost_req, ROUND_SIZE),
            "sdk_func_name" : self.function_name,
            "run_status" : self.run_status,
            "sdk_test_start_time" : self.sdk_api_start_time
        }
        global time_cost_detail_list
        time_cost_detail_list.append(row_dict)

    @staticmethod
    def output_sdk_cost_list(data_list, format_str="", only_brief=False):

        TimeCost.output_sdk_header(format_str, only_brief)

        for dict_data in data_list:
            TimeCost.output_sdk_cost(dict_data, format_str, only_brief)

    @staticmethod
    def output_sdk_header(format_str, only_brief):
        delay_server_api_cost = "delay(server_api_cost){format_str}".format(format_str=format_str)
        delay_server_req_cost = "delay(server_req_cost){format_str}".format(format_str=format_str)
        sdk_api_cost = "sdk_api_cost{format_str}".format(format_str=format_str)

        sdk_test_start_time = "sdk_test_start_time{format_str}".format(format_str=format_str)
        sdk_func_name = "sdk_func_name{format_str}".format(format_str=format_str)
        run_status = "run_status{format_str}".format(format_str=format_str)

        if only_brief:
            print(delay_server_api_cost,
                  delay_server_req_cost,
                  sdk_api_cost)
        else:
            print(
                  delay_server_api_cost,
                  delay_server_req_cost,
                  sdk_api_cost,
                  sdk_test_start_time,
                  sdk_func_name,
                  run_status)

    @staticmethod
    def output_sdk_cost(dict_data, format_str, only_brief):
        sdk_test_start_time = dict_data.get("sdk_test_start_time", "")
        if sdk_test_start_time:
            sdk_test_start_time_desc = "{sdk_test_start_time}{format_str}".format(
                                sdk_test_start_time=dict_data["sdk_test_start_time"],
                                format_str=format_str)
        else:
            sdk_test_start_time_desc = ""

        sdk_api_delay_desc ="{sdk_api_delay}({server_api_cost}){format_str}".format(
                        sdk_api_delay=dict_data["sdk_api_delay"],
                        server_api_cost=dict_data["server_api_cost"],
                        format_str=format_str)

        sdk_req_delay_desc = "{sdk_req_delay}({server_req_cost}){format_str}".format(
                        sdk_req_delay=dict_data["sdk_req_delay"],
                        server_req_cost=dict_data["server_req_cost"],
                        format_str=format_str)

        sdk_api_cost_desc = "{sdk_api_cost}{format_str}".format(
                        sdk_api_cost=dict_data["sdk_api_cost"],
                        format_str=format_str)

        sdk_func_name = dict_data.get("sdk_func_name", None)
        if sdk_func_name:
            sdk_func_name_desc = "{sdk_func_name}{format_str}".format(
                sdk_func_name=dict_data["sdk_func_name"],
                format_str=format_str)
        else:
            sdk_func_name_desc = ""

        run_status = dict_data.get("run_status", None)
        if run_status:
            run_status_desc = "{run_status}{format_str}".format(
                run_status=dict_data["run_status"],
                format_str=format_str)
        else:
            run_status_desc = ""


        if only_brief:
            print(
                sdk_api_delay_desc,
                sdk_req_delay_desc,
                sdk_api_cost_desc,
            )
        else:
            print(
                sdk_api_delay_desc,
                sdk_req_delay_desc,
                sdk_api_cost_desc,
                sdk_test_start_time_desc,
                sdk_func_name_desc,
                run_status_desc
            )


    @staticmethod
    def output_sort_cost(by_key_name, is_sorted=False):
        global time_cost_detail_list

        if is_sorted:
            output_list = sorted(time_cost_detail_list, key=lambda e: e.__getitem__(by_key_name), reverse=True)
        else:
            output_list = time_cost_detail_list

        TimeCost.output_sdk_cost_list(data_list=output_list, format_str="\t", only_brief=False)

    @staticmethod
    def output_average_cost():
        global time_cost_detail_list
        global count_offset
        sum_final = {}
        average_final = {}
        sum_key_list = ["sdk_api_cost", "server_api_cost", "server_req_cost", "sdk_api_delay", "sdk_req_delay"]
        if len(time_cost_detail_list):
            average_count = len(time_cost_detail_list) + count_offset
            for key_name in sum_key_list:
                sum_final[key_name] = sum(row[key_name] for row in time_cost_detail_list)
                average_final[key_name] = round(sum_final[key_name] / average_count, ROUND_SIZE)

        print("api counts :", average_count, count_offset)
        #TimeCost.output_sdk_cost_list(data_list=[sum_final], only_brief=True)
        TimeCost.output_sdk_cost_list(data_list=[average_final], only_brief=True)


class RestfulTestCaseSeq:

    def __init__(self):
        pass

    def test_generic(self):
        generic_client = GenericClient(api_key=g_api_key, secret_key=g_secret_key, performance_test=True)
        # case get_exchange_symbol_list
        tc = TimeCost(function_name=generic_client.get_exchange_timestamp.__name__)
        result, tc.server_req_cost, tc.server_api_cost = generic_client.get_exchange_timestamp()
        tc.run_status = RunStatus.SUCCESS if result and result > 0 else RunStatus.FAILED
        tc.add_record()

        # case get_exchange_currencies
        tc = TimeCost(function_name=generic_client.get_exchange_currencies.__name__)
        result, tc.server_req_cost, tc.server_api_cost = generic_client.get_exchange_currencies()
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

        # case get_exchange_symbols
        tc = TimeCost(function_name=generic_client.get_exchange_symbols.__name__)
        result, tc.server_req_cost, tc.server_api_cost = generic_client.get_exchange_symbols()
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

        # case get_reference_currencies
        tc = TimeCost(function_name=generic_client.get_reference_currencies.__name__)
        result, tc.server_req_cost, tc.server_api_cost = generic_client.get_reference_currencies()
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

        # case get_system_status
        tc = TimeCost(function_name=generic_client.get_system_status.__name__)
        result, tc.server_req_cost, tc.server_api_cost = generic_client.get_system_status()
        tc.run_status = RunStatus.SUCCESS if result and result.get("page") and result.get("components") else RunStatus.FAILED
        tc.add_record()

    def test_market(self):
        market_client = MarketClientPerformance(api_key=g_api_key, secret_key=g_secret_key)
        common_market_symbol = "btcusdt"

        #case get_candlestick
        tc = TimeCost(function_name=market_client.get_candlestick.__name__)
        result, tc.server_req_cost, tc.server_api_cost = market_client.get_candlestick(symbol=common_market_symbol,
                                                                                       period=CandlestickInterval.MIN1,
                                                                                       size=150)
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

        # case get_pricedepth
        tc = TimeCost(function_name=market_client.get_pricedepth.__name__)
        result, tc.server_req_cost, tc.server_api_cost = market_client.get_pricedepth(symbol=common_market_symbol, depth_type=DepthStep.STEP0, depth_size=20)
        tc.run_status = RunStatus.SUCCESS if result and len(result.bids) else RunStatus.FAILED
        tc.add_record()

        # case get_market_detail
        tc = TimeCost(function_name=market_client.get_market_trade.__name__)
        result, tc.server_req_cost, tc.server_api_cost = market_client.get_market_detail(symbol=common_market_symbol)
        tc.run_status = RunStatus.SUCCESS if result and result.id else RunStatus.FAILED
        tc.add_record()

        # case get_market_trade
        tc = TimeCost(function_name = market_client.get_market_trade.__name__)
        result, tc.server_req_cost, tc.server_api_cost = market_client.get_market_trade(symbol=common_market_symbol)
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()


        # case get_history_trade
        tc = TimeCost(function_name=market_client.get_history_trade.__name__)
        result, tc.server_req_cost, tc.server_api_cost = market_client.get_history_trade(symbol=common_market_symbol)
        tc.run_status = RunStatus.SUCCESS if result and result.count else RunStatus.FAILED
        tc.add_record()

        # case get_market_detail_merged
        tc = TimeCost(function_name=market_client.get_market_detail_merged.__name__)
        result, tc.server_req_cost, tc.server_api_cost = market_client.get_market_detail_merged(symbol=common_market_symbol)
        tc.run_status = RunStatus.SUCCESS if result and len(result.bid) and len(result.ask) else RunStatus.FAILED
        tc.add_record()

        # case get_market_tickers
        tc = TimeCost(function_name=market_client.get_market_tickers.__name__)
        result, tc.server_req_cost, tc.server_api_cost = market_client.get_market_tickers()
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

    def test_wallet(self):
        wallet_client = WalletClient(api_key=g_api_key, secret_key=g_secret_key, performance_test=True)

        # case get_account_deposit_address
        tc = TimeCost(function_name=wallet_client.get_account_deposit_address.__name__)
        result, tc.server_req_cost, tc.server_api_cost = wallet_client.get_account_deposit_address(currency="usdt")
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

        # case get_account_withdraw_quota
        tc = TimeCost(function_name=wallet_client.get_account_withdraw_quota.__name__)
        result, tc.server_req_cost, tc.server_api_cost = wallet_client.get_account_withdraw_quota(currency="usdt")
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()


        # case get_sub_user_deposit_address
        tc = TimeCost(function_name=wallet_client.get_sub_user_deposit_address.__name__)
        result, tc.server_req_cost, tc.server_api_cost = wallet_client.get_sub_user_deposit_address(sub_uid=g_sub_uid, currency="btc")
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

        # case get_sub_user_deposit_history
        tc = TimeCost(function_name=wallet_client.get_sub_user_deposit_history.__name__)
        result, tc.server_req_cost, tc.server_api_cost = wallet_client.get_sub_user_deposit_history(sub_uid=g_sub_uid)
        tc.run_status = RunStatus.SUCCESS if result else RunStatus.FAILED
        tc.add_record()

        # case get_deposit_withdraw
        tc = TimeCost(function_name=wallet_client.get_deposit_withdraw.__name__)
        result, tc.server_req_cost, tc.server_api_cost = wallet_client.get_deposit_withdraw(op_type=DepositWithdraw.DEPOSIT, currency=None, from_id=1, size=10, direct=QueryDirection.PREV)
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

        # case post_create_withdraw
        tc = TimeCost(function_name=wallet_client.post_create_withdraw.__name__)
        record_id, tc.server_req_cost, tc.server_api_cost = wallet_client.post_create_withdraw(address=withdraw_address,
                                                     amount=2, currency="usdt", fee=0,
                                                     chain="trc20usdt", address_tag=None)
        tc.run_status = RunStatus.SUCCESS if record_id and record_id > 0 else RunStatus.FAILED
        tc.add_record()

        # case post_cancel_withdraw
        tc = TimeCost(function_name=wallet_client.post_cancel_withdraw.__name__)
        record_id, tc.server_req_cost, tc.server_api_cost = wallet_client.post_cancel_withdraw(withdraw_id=record_id)
        tc.run_status = RunStatus.SUCCESS if record_id and record_id > 0 else RunStatus.FAILED
        tc.add_record()

    def test_cross_margin(self):
        margin_client = MarginClient(api_key=g_api_key, secret_key=g_secret_key, performance_test=True)

        # case post_cross_margin_transfer_in
        tc = TimeCost(function_name=margin_client.post_cross_margin_transfer_in.__name__)
        result, tc.server_req_cost, tc.server_api_cost = margin_client.post_cross_margin_transfer_in(currency=loan_currency, amount=loan_amount)
        tc.run_status = RunStatus.SUCCESS if result else RunStatus.FAILED
        tc.add_record()

        # case get_cross_margin_account_balance
        tc = TimeCost(function_name=margin_client.get_cross_margin_account_balance.__name__)
        result, tc.server_req_cost, tc.server_api_cost = margin_client.get_cross_margin_account_balance()
        tc.run_status = RunStatus.SUCCESS if result and result.id else RunStatus.FAILED
        tc.add_record()

        # case get_cross_margin_loan_info
        tc = TimeCost(function_name=margin_client.get_cross_margin_loan_info.__name__)
        result, tc.server_req_cost, tc.server_api_cost = margin_client.get_cross_margin_loan_info()
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()


        # case post_cross_margin_create_loan_orders
        tc = TimeCost(function_name=margin_client.post_cross_margin_create_loan_orders.__name__)
        result, tc.server_req_cost, tc.server_api_cost = margin_client.post_cross_margin_create_loan_orders(currency=loan_currency, amount=loan_amount)
        tc.run_status = RunStatus.SUCCESS if result and int(result) > 0 else RunStatus.FAILED
        tc.add_record()

        time.sleep(3) # wait for interest amount
        # case get_cross_margin_loan_orders
        tc = TimeCost(function_name=margin_client.get_cross_margin_loan_orders.__name__)
        result, tc.server_req_cost, tc.server_api_cost = margin_client.get_cross_margin_loan_orders(currency=loan_currency, state=LoanOrderState.ACCRUAL)
        cross_loan_accrual_order_list = result
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

        # case post_cross_margin_loan_order_repay
        interest_amount = 0.0
        if cross_loan_accrual_order_list and len(cross_loan_accrual_order_list):
            for loan_order in cross_loan_accrual_order_list:
                tc = TimeCost(function_name=margin_client.post_cross_margin_loan_order_repay.__name__)
                repay_amount = float(loan_order.loan_balance) + float(loan_order.interest_balance)
                interest_amount = interest_amount + float(loan_order.interest_balance)
                result, tc.server_req_cost, tc.server_api_cost = margin_client.post_cross_margin_loan_order_repay(order_id=loan_order.id, amount=repay_amount)
                tc.run_status = RunStatus.SUCCESS if result else RunStatus.FAILED
                tc.add_record()

        time.sleep(2)  # wait for repay result
        # case post_cross_margin_transfer_out
        tc = TimeCost(function_name=margin_client.post_cross_margin_transfer_out.__name__)
        result, tc.server_req_cost, tc.server_api_cost = margin_client.post_cross_margin_transfer_out(currency=loan_currency, amount=loan_amount-interest_amount)
        tc.run_status = RunStatus.SUCCESS if result else RunStatus.FAILED
        tc.add_record()

    def test_margin(self):
        margin_client = MarginClient(api_key=g_api_key, secret_key=g_secret_key, performance_test=True)

        # case post_transfer_in_margin
        tc = TimeCost(function_name=margin_client.post_transfer_in_margin.__name__)
        result, tc.server_req_cost, tc.server_api_cost = margin_client.post_transfer_in_margin(symbol=loan_symbol, currency=loan_currency, amount=loan_amount)
        tc.run_status = RunStatus.SUCCESS if result else RunStatus.FAILED
        tc.add_record()

        # case get_margin_loan_info
        tc = TimeCost(function_name=margin_client.get_margin_loan_info.__name__)
        result, tc.server_req_cost, tc.server_api_cost = margin_client.get_margin_loan_info(symbols="btcusdt,ethusdt,eosusdt,"+loan_symbol)
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

        # case get_margin_loan_orders
        tc = TimeCost(function_name=margin_client.get_margin_loan_orders.__name__)
        result, tc.server_req_cost, tc.server_api_cost = margin_client.get_margin_loan_orders(symbol=loan_symbol)
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

        # case post_create_margin_order
        tc = TimeCost(function_name=margin_client.post_create_margin_order.__name__)
        result, tc.server_req_cost, tc.server_api_cost = margin_client.post_create_margin_order(
            symbol=loan_symbol, currency=loan_currency, amount=loan_amount)
        tc.run_status = RunStatus.SUCCESS if result and int(result) > 0 else RunStatus.FAILED
        tc.add_record()

        time.sleep(3)  # wait for interest amount
        # case get_margin_loan_orders
        tc = TimeCost(function_name=margin_client.get_margin_loan_orders.__name__)
        result, tc.server_req_cost, tc.server_api_cost = margin_client.get_margin_loan_orders(
            symbol=loan_symbol, states=LoanOrderState.ACCRUAL)
        cross_loan_accrual_order_list = result
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

        # case post_repay_margin_order
        interest_amount = 0.0
        if cross_loan_accrual_order_list and len(cross_loan_accrual_order_list):
            for loan_order in cross_loan_accrual_order_list:
                tc = TimeCost(function_name=margin_client.post_repay_margin_order.__name__)
                repay_amount = float(loan_order.loan_balance) + float(loan_order.interest_balance)
                interest_amount = interest_amount + float(loan_order.interest_balance)
                result, tc.server_req_cost, tc.server_api_cost = margin_client.post_repay_margin_order(
                    loan_id=loan_order.id, amount=repay_amount)
                tc.run_status = RunStatus.SUCCESS if result else RunStatus.FAILED
                tc.add_record()

        time.sleep(2)  # wait for repay result

        # case post_transfer_out_margin
        tc = TimeCost(function_name=margin_client.post_transfer_out_margin.__name__)
        result, tc.server_req_cost, tc.server_api_cost = margin_client.post_transfer_out_margin(symbol=loan_symbol,
                                                                                               currency=loan_currency,
                                                                                               amount=loan_amount - interest_amount)
        tc.run_status = RunStatus.SUCCESS if result else RunStatus.FAILED
        tc.add_record()

    def test_trade_fee(self):
        trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key, performance_test=True)

        # case get_feerate
        tc = TimeCost(function_name=trade_client.get_feerate.__name__)
        result, tc.server_req_cost, tc.server_api_cost = trade_client.get_feerate(symbols="htusdt,btcusdt,eosusdt,"+trade_symbol)
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

        # case get_transact_feerate
        tc = TimeCost(function_name=trade_client.get_transact_feerate.__name__)
        result, tc.server_req_cost, tc.server_api_cost = trade_client.get_transact_feerate(symbols="htusdt,btcusdt,eosusdt,"+trade_symbol)
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

    def test_trade_order(self):
        trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key, performance_test=True)

        client_order_id = "test_"+str(round(time.time()))+"_id"
        print("client order id : ", client_order_id)
        # case create_order
        tc = TimeCost(function_name=trade_client.create_order.__name__)
        result, tc.server_req_cost, tc.server_api_cost = trade_client.order_id = trade_client.create_order(symbol=trade_symbol,
                                     account_id=g_account_id,
                                     order_type=OrderType.BUY_LIMIT,
                                     source=OrderSource.API,
                                     amount=55,
                                     price=0.1,
                                     client_order_id=client_order_id,
                                     stop_price=0.08,
                                     operator="gte")
        order_id_tmp = result
        tc.run_status = RunStatus.SUCCESS if result else RunStatus.FAILED
        tc.add_record()

        # case get_order
        tc = TimeCost(function_name=trade_client.get_order.__name__)
        result, tc.server_req_cost, tc.server_api_cost = trade_client.order_id = trade_client.get_order(order_id=order_id_tmp)
        tc.run_status = RunStatus.SUCCESS if result else RunStatus.FAILED
        tc.add_record()

        # case get_order_by_client_order_id
        tc = TimeCost(function_name=trade_client.get_order_by_client_order_id.__name__)
        result, tc.server_req_cost, tc.server_api_cost = trade_client.order_id = trade_client.get_order_by_client_order_id(client_order_id=client_order_id)
        tc.run_status = RunStatus.SUCCESS if result else RunStatus.FAILED
        tc.add_record()

        # case get_open_orders
        tc = TimeCost(function_name=trade_client.get_open_orders.__name__)
        result, tc.server_req_cost, tc.server_api_cost = trade_client.order_id = trade_client.get_open_orders(symbol=trade_symbol, account_id=g_account_id)
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

        # case get_orders
        tc = TimeCost(function_name=trade_client.get_orders.__name__)
        result, tc.server_req_cost, tc.server_api_cost = trade_client.order_id = trade_client.get_orders(symbol=trade_symbol, order_type=OrderType.BUY_LIMIT, order_state=OrderState.SUBMITTED)
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

        # case cancel_order
        tc = TimeCost(function_name=trade_client.cancel_order.__name__)
        result, tc.server_req_cost, tc.server_api_cost = trade_client.order_id = trade_client.cancel_order(symbol=trade_symbol, order_id=order_id_tmp)
        tc.run_status = RunStatus.SUCCESS if result else RunStatus.FAILED
        tc.add_record()

    def test_trade_create_cancel_orders(self):
        trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key, performance_test=True)
        client_order_id_header = "test_"+str(round(time.time()))+"_id"
        client_order_id_eos_01 = client_order_id_header + "01"
        client_order_id_eos_02 = client_order_id_header + "02"
        client_order_id_eos_03 = client_order_id_header + "03"

        buy_limit_eos_01 = {
            "account_id": g_account_id,
            "symbol": trade_symbol,
            "order_type": OrderType.BUY_LIMIT,
            "source": OrderSource.API,
            "amount": 50,
            "price": 0.12,
            "client_order_id": client_order_id_eos_01
        }

        buy_limit_eos_02 = {
            "account_id": g_account_id,
            "symbol": trade_symbol,
            "order_type": OrderType.BUY_LIMIT,
            "source": OrderSource.API,
            "amount": 7,
            "price": 0.80,
            "client_order_id": client_order_id_eos_02
        }

        buy_limit_eos_03 = {
            "account_id": g_account_id,
            "symbol": trade_symbol,
            "order_type": OrderType.BUY_LIMIT,
            "source": OrderSource.API,
            "amount": 20,
            "price": 0.252,
            "client_order_id": client_order_id_eos_03
        }

        order_config_list = [
            buy_limit_eos_01,
            buy_limit_eos_02,
            buy_limit_eos_03
        ]

        # case batch_create_order
        tc = TimeCost(function_name=trade_client.batch_create_order.__name__)
        create_result, tc.server_req_cost, tc.server_api_cost = trade_client.batch_create_order(order_config_list=order_config_list)
        tc.run_status = RunStatus.SUCCESS if create_result else RunStatus.FAILED
        tc.add_record()

        # time.sleep(1)  # need wait

        order_id_list = []
        if create_result and len(create_result):
            for item in create_result:
                order_id_list.append(item.order_id)

            # case batch_create_order
            tc = TimeCost(function_name=trade_client.batch_create_order.__name__)
            result, tc.server_req_cost, tc.server_api_cost = trade_client.cancel_orders(symbol=trade_symbol, order_id_list=order_id_list)
            tc.run_status = RunStatus.SUCCESS if result else RunStatus.FAILED
            tc.add_record()

    def test_trade_create_cancel_open_orders(self):
        trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key, performance_test=True)
        order_config_list = []
        for i in range(6):
            buy_limit_item = {
                "account_id": g_account_id,
                "symbol": trade_symbol,
                "order_type": OrderType.BUY_LIMIT,
                "source": OrderSource.API,
                "amount": 50,
                "price": 0.12 + round(i/100, 3),
            }
            order_config_list.append(buy_limit_item)

        # case batch_create_order
        tc = TimeCost(function_name=trade_client.batch_create_order.__name__)
        create_result, tc.server_req_cost, tc.server_api_cost = trade_client.batch_create_order(order_config_list=order_config_list)
        print(type(create_result), type(tc.server_req_cost), type(tc.server_api_cost))
        tc.run_status = RunStatus.SUCCESS if create_result else RunStatus.FAILED
        tc.add_record()


        # case cancel_open_orders
        tc = TimeCost(function_name=trade_client.cancel_open_orders.__name__)
        result, tc.server_req_cost, tc.server_api_cost = trade_client.cancel_open_orders(account_id=g_account_id)
        tc.run_status = RunStatus.SUCCESS if result else RunStatus.FAILED
        tc.add_record()


    def test_trade_match_result(self):
        trade_client = TradeClient(api_key=g_api_key, secret_key=g_secret_key, performance_test=True)

        tc = TimeCost(function_name=trade_client.create_order.__name__)
        created_order_id, tc.server_req_cost, tc.server_api_cost = trade_client.create_order(
            symbol=trade_symbol, account_id=g_account_id, order_type=OrderType.BUY_MARKET, source=OrderSource.API, amount=5.0, price=None)
        print("create order id for match result: ", created_order_id)
        tc.run_status = RunStatus.SUCCESS if created_order_id else RunStatus.FAILED
        tc.add_record()
        time.sleep(2)  # match result is not ready, need sleep a time

        # case get_match_results_by_order_id
        tc = TimeCost(function_name=trade_client.get_match_results_by_order_id.__name__)
        result, tc.server_req_cost, tc.server_api_cost = trade_client.get_match_results_by_order_id(order_id=created_order_id)
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

        # case get_match_result
        tc = TimeCost(function_name=trade_client.get_match_result.__name__)
        result, tc.server_req_cost, tc.server_api_cost = trade_client.get_match_result(
            symbol=trade_symbol, size=10)
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

    def test_account(self):
        account_client = AccountClientPerformance(api_key=g_api_key, secret_key=g_secret_key, performance_test=True)
        # case get_accounts
        tc = TimeCost(function_name=account_client.get_accounts.__name__)
        result, tc.server_req_cost, tc.server_api_cost = account_client.get_accounts()
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

        # case get_account_balance
        tc = TimeCost(function_name=account_client.get_account_balance.__name__)
        result, tc.server_req_cost, tc.server_api_cost = account_client.get_account_balance()
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

        # case get_account_balance_by_subuid
        tc = TimeCost(function_name=account_client.get_account_balance_by_subuid.__name__)
        result, tc.server_req_cost, tc.server_api_cost = account_client.get_account_balance_by_subuid(sub_uid=g_sub_uid)
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

        # case get_account_by_type_and_symbol
        tc = TimeCost(function_name=account_client.get_account_by_type_and_symbol.__name__)
        result, tc.server_req_cost, tc.server_api_cost = account_client.get_account_by_type_and_symbol(account_type=AccountType.SPOT, symbol=None)
        account_id_spot = result.id if result else None
        tc.run_status = RunStatus.SUCCESS if result and result.id and account_id_spot else RunStatus.FAILED
        tc.add_record()


        # case get_account_history
        tc = TimeCost(function_name=account_client.get_account_history.__name__)
        result, tc.server_req_cost, tc.server_api_cost = account_client.get_account_history(account_id=account_id_spot)
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

        # case get_account_ledger
        tc = TimeCost(function_name=account_client.get_account_ledger.__name__)
        result, tc.server_req_cost, tc.server_api_cost = account_client.get_account_ledger(account_id=account_id_spot)
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

        # case get_aggregated_subuser_balance
        tc = TimeCost(function_name=account_client.get_aggregated_subuser_balance.__name__)
        result, tc.server_req_cost, tc.server_api_cost = account_client.get_aggregated_subuser_balance()
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

        # case get_balance
        tc = TimeCost(function_name=account_client.get_balance.__name__)
        result, tc.server_req_cost, tc.server_api_cost = account_client.get_balance(account_id=account_id_spot)
        tc.run_status = RunStatus.SUCCESS if result and len(result) else RunStatus.FAILED
        tc.add_record()

        # case post_sub_uid_management
        tc = TimeCost(function_name=account_client.post_sub_uid_management.__name__ + "_lock")
        result, tc.server_req_cost, tc.server_api_cost = account_client.post_sub_uid_management(sub_uid=g_sub_uid, action=SubUidAction.LOCK)
        tc.run_status = RunStatus.SUCCESS if result and (result.userState == SubUidState.LOCK) else RunStatus.FAILED
        tc.add_record()

        # case post_sub_uid_management
        tc = TimeCost(function_name=account_client.post_sub_uid_management.__name__ + "_unlock")
        result, tc.server_req_cost, tc.server_api_cost = account_client.post_sub_uid_management(sub_uid=g_sub_uid, action=SubUidAction.UNLOCK)
        tc.run_status = RunStatus.SUCCESS if result and (result.userState == SubUidState.NORMAL) else RunStatus.FAILED
        tc.add_record()

        # case transfer_between_futures_and_pro
        tc = TimeCost(function_name=account_client.transfer_between_futures_and_pro.__name__)
        result, tc.server_req_cost, tc.server_api_cost = account_client.transfer_between_futures_and_pro(currency="trx", amount=200, transfer_type=TransferFuturesPro.TO_FUTURES)
        time.sleep(2)
        result, tc.server_req_cost, tc.server_api_cost = account_client.transfer_between_futures_and_pro(currency="trx", amount=200, transfer_type=TransferFuturesPro.TO_PRO)
        tc.run_status = RunStatus.SUCCESS if result and result > 0 else RunStatus.FAILED
        tc.add_record()

        # case transfer_between_parent_and_subuser
        tc = TimeCost(function_name=account_client.transfer_between_parent_and_subuser.__name__)
        result, tc.server_req_cost, tc.server_api_cost = account_client.transfer_between_parent_and_subuser(sub_uid=g_sub_uid, currency="usdt", amount=10, transfer_type=TransferMasterType.OUT)
        result, tc.server_req_cost, tc.server_api_cost = account_client.transfer_between_parent_and_subuser(sub_uid=g_sub_uid, currency="usdt", amount=10, transfer_type=TransferMasterType.IN)
        tc.run_status = RunStatus.SUCCESS if result and result > 0 else RunStatus.FAILED
        tc.add_record()


    def test_etf(self):
        etf_client = EtfClient(api_key=g_api_key, secret_key=g_secret_key, performance_test=True)
        # case get_etf_swap_config
        tc = TimeCost(function_name=etf_client.get_etf_swap_config.__name__)
        result, tc.server_req_cost, tc.server_api_cost = etf_client.get_etf_swap_config(etf_name="hb10")
        tc.run_status = RunStatus.SUCCESS if result else RunStatus.FAILED
        tc.add_record()

        # case get_etf_swap_list
        tc = TimeCost(function_name=etf_client.get_etf_swap_list.__name__)
        result, tc.server_req_cost, tc.server_api_cost = etf_client.get_etf_swap_list(etf_name="hb10", offset=0, size=10)
        tc.run_status = RunStatus.SUCCESS if len(result) >= 0 else RunStatus.FAILED
        tc.add_record()




if __name__ == "__main__":
    test_case = RestfulTestCaseSeq()
    test_case.test_generic()
    test_case.test_market()
    test_case.test_wallet()
    test_case.test_cross_margin()
    test_case.test_margin()
    test_case.test_account()
    test_case.test_etf()
    test_case.test_cross_margin()
    test_case.test_trade_fee()
    test_case.test_trade_match_result()
    test_case.test_trade_order()
    test_case.test_trade_create_cancel_orders()
    test_case.test_trade_create_cancel_open_orders()

    print("\n\n==================api execute sequence=========================")
    TimeCost.output_sort_cost(by_key_name="", is_sorted=False)

    print("\n\n======================order by api delay time desc=====================")
    TimeCost.output_sort_cost(by_key_name="sdk_api_delay", is_sorted=True)

    print("\n\n======================average cost/delay time=====================")
    TimeCost.output_average_cost()

















import unittest
import huobi
from unittest import mock
from huobi.utils import *



class TestApi(unittest.TestCase):
    def test_request(self):
        builder = UrlParamsBuilder()
        huobi.utils.api_signature.utc_now = mock.Mock(return_value="123")
        create_signature("123", "456", "GET",  "http://host/url", builder)
        self.assertEqual("?AccessKeyId=123&SignatureVersion=2&SignatureMethod=HmacSHA256&Timestamp=123&Signature=Hhiaq8xYQPiBZOyWV37MdQutLo4f0ZOHiJtG3p%2BnILc%3D", builder.build_url())
        return


if __name__ == "__main__":
    unittest.main()



class Candlestick:
    """
    The candlestick/kline data.

    :member
        id : keep the original timestamp
        timestamp: The UNIX formatted timestamp in UTC.
        high: The high price.
        low: The low price.
        open: The opening price.
        close: The closing price.
        amount: The aggregated trading volume in USDT.
        count: The number of completed trades. it returns 0 when get ETF candlestick
        vol: The trading volume in base currency.

    """

    def __init__(self):
        self.id = 0
        #self.timestamp = 0
        self.high = 0.0
        self.low = 0.0
        self.open = 0.0
        self.close = 0.0
        self.amount = 0.0
        self.count = 0
        self.vol = 0.0  #self.volume = 0.0


    def print_object(self, format_data=""):
        from huobi.utils.print_mix_object import PrintBasic
        PrintBasic.print_basic(self.id, format_data + "Id")
        #PrintBasic.print_basic(self.timestamp, format_data + "Unix Time")
        PrintBasic.print_basic(self.high, format_data + "High")
        PrintBasic.print_basic(self.low, format_data + "Low")
        PrintBasic.print_basic(self.open, format_data + "Open")
        PrintBasic.print_basic(self.close, format_data + "Close")
        PrintBasic.print_basic(self.count, format_data + "Count")
        PrintBasic.print_basic(self.amount, format_data + "Amount")
        PrintBasic.print_basic(self.vol, format_data + "Volume")


import time

from huobi.model.market import *
from huobi.utils import *
from huobi.connection.subscribe_client import SubscribeClient


class SubTradeDetailService:
    def __init__(self, params):
        self.params = params

    def subscribe(self, callback, error_handler, **kwargs):
        symbol_list = self.params["symbol_list"]

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(trade_detail_channel(symbol))
                time.sleep(0.01)

        def parse(dict_data):
            tick = dict_data.get("tick", {})
            trade_detail_event = default_parse(tick, TradeDetailEvent, TradeDetail)
            trade_detail_event.ch = dict_data.get("ch", "")
            return trade_detail_event

        SubscribeClient(**kwargs).execute_subscribe_v1(subscription,
                                            parse,
                                            callback,
                                            error_handler)



